/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common;

// ToolBox imports
import org.enhydra.tool.common.event.CancelEvent;
import org.enhydra.tool.common.event.CancelListener;
import org.enhydra.tool.common.event.ProgressEvent;
import org.enhydra.tool.common.event.ProgressListener;

// Standard imports
import java.util.ArrayList;
import java.util.Arrays;

//
public class ProgressBuilder implements CancelListener {
    private ProgressListener[] progressListeners = new ProgressListener[0];
    private static boolean fresh = true;

    public ProgressBuilder() {
    }


    /**
     * Make the thread stale so it stops running gracefully.
     */
    public void setFresh(boolean b) {
        fresh = b;
    }

    public boolean isFresh() {
        return fresh;
    }

    public ProgressListener[] getProgressListeners() {
        return progressListeners;
    }

    public void addProgressListener(ProgressListener l) {
    	ArrayList list = null;
    	list = new ArrayList(Arrays.asList(progressListeners));
    	if (! list.contains(l)) {
            list.add(l);
        }
        list.trimToSize();
        progressListeners = new ProgressListener[list.size()];
        progressListeners = (ProgressListener[]) list.toArray(progressListeners);
        list.clear();
    }

    public void removeProgressListener(ProgressListener l) {
    	ArrayList list = null;
    	list = new ArrayList(Arrays.asList(progressListeners));
    	if (list.contains(l)) {
            list.remove(l);
        }
        list.trimToSize();
        progressListeners = new ProgressListener[list.size()];
        progressListeners = (ProgressListener[]) list.toArray(progressListeners);
        list.clear();
    }

    public void refreshProgress(int progress, String message) {
        if (progressListeners.length == 0) {

            // done
        } else {
            ProgressEvent event = null;

            event = new ProgressEvent(this, progress, message);
            for (int i = 0; i < progressListeners.length; i++) {
                progressListeners[i].onProgress(event);
            }
        }
    }

    public void onCancel(CancelEvent event) {
        setFresh(false);
    }

}
