/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.bridge;

// XMLC imports
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.XMLCException;

// Standard imports
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

//
public class ParserV1 implements Parser {

    // strings not to be resourced
    private final String    ERROR_REPORTER_CLASS =
        "org.enhydra.xml.xmlc.compiler.ErrorReporter"; // nores
    private final String    PARSE_CLASS =
        "org.enhydra.xml.xmlc.compiler.Parse"; // nores
    private final String    PARSE_METHOD = "parse"; // nores

    //
    private Parse           parse = null;
    private PrintWriter     traceWriter = null;
    private MetaDataHandler handler = null;

    public ParserV1(PrintWriter trace, MetaDataHandler metaData) {
        traceWriter = trace;
        handler = metaData;
        Class         parseClass = null;
        Constructor[] cons = new Constructor[0];
        Object[]      values = new Object[2];

        try {
            parseClass = Class.forName(PARSE_CLASS);
            cons = parseClass.getConstructors();
            for (int i = 0; i < cons.length; i++) {
                if (cons[i].getParameterTypes().length == 2) {
                    values[0] = createErrorReporter(traceWriter);
                    if (handler.getPrintParseInfo()) {
                        values[1] = traceWriter;
                    } else {
                        values[1] = null;
                    }
                    parse = (Parse) cons[i].newInstance(values);
                    break;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public MetaDataHandler getMetaDataHandler() {
        return handler;
    }

    public PrintWriter getTraceWriter() {
        return traceWriter;
    }

    public XMLCDocument parse() throws XMLCException {
        XMLCDocument doc = null;

        doc = (XMLCDocument) callParse();
        return doc;
    }

    private Object createErrorReporter(PrintWriter writer) {
        Object      reporter = null;
        Constructor con = null;
        Class       reportClass = null;
        Class[]     types = new Class[1];
        Object[]    values = new Object[1];

        types[0] = writer.getClass();
        try {
            reportClass = Class.forName(ERROR_REPORTER_CLASS);
            con = reportClass.getConstructor(types);
            values[0] = writer;
            reporter = con.newInstance(values);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return reporter;
    }

    private Object callParse() throws XMLCException {
        Object doc = null;
        Method meths[] = parse.getClass().getDeclaredMethods();
        Method parseMethod = null;

        for (int i = 0; i < meths.length; i++) {
            if (meths[i].getName().equals(PARSE_METHOD)) {
                parseMethod = meths[i];
                break;
            }
        }
        if (parseMethod != null) {
            Object params[] = new Object[1];

            params[0] = handler.getMetaData();
            try {
                doc = parseMethod.invoke(parse, params);
            } catch (java.lang.IllegalAccessException e) {
                e.printStackTrace();
            } catch (java.lang.reflect.InvocationTargetException e) {
                if (e.getTargetException() instanceof XMLCException) {
                    throw (XMLCException) e.getTargetException();
                } else {
                    XMLCException xe =
                        new XMLCException(e.getTargetException().toString());

                    e.getTargetException().printStackTrace();
                    throw xe;
                }
            }
        }
        return doc;
    }

}
