/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

package org.enhydra.kelp.common.bridge;

// XMLC imports - 1.x and 2.x
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.w3c.dom.*;

// Kelp imports
import org.enhydra.kelp.common.Constants;

// Standard imports
import java.io.*;
import java.util.ResourceBundle;
import java.util.Vector;

abstract public class PrintInfo {
    // string not to be resourced
    private final String ARROW = " => "; // nores
    private static ResourceBundle res = ResourceBundle.getBundle("org.enhydra.kelp.common.Res"); // nores

    /**
     * Table URLs.
     */
    protected Vector urls = new Vector();

    /**
     * Table of element ids.
     */
    protected Vector ids = new Vector();

    /**
     * Construct document info from document.
     */
    public PrintInfo(Document doc, XMLCDocument xmlcDoc) {
        getNodeInfo(doc, xmlcDoc);
    }

    /**
     * Print document information.
     */
    public void printInfo(PrintWriter out) {
        out.println(res.getString("Element_IDs"));
        for (int i = 0; i < ids.size(); i++) {
            out.println(Constants.TAB4 + ids.elementAt(i).toString());
        }
        out.println(res.getString("Document_URLs"));
        for (int i = 0; i < urls.size(); i++) {
            out.println(Constants.TAB4 + urls.elementAt(i).toString());
        }
    }

    abstract protected void getElementURLs(Element element, XMLCDocument xmlcDoc);


    /**
     * Recursively scan nodes nodes and accumulate information.
     */
    private void getNodeInfo(Node node, XMLCDocument xmlcDoc) {
        if (node instanceof Element) {
            Element elem = (Element)node;
            getElementURLs(elem, xmlcDoc);
            String id = xmlcDoc.getElementId(elem);
            if ((id != null) && (id.length() > 0)) {
                ids.addElement(id + ARROW
                               + xmlcDoc.nodeClassToInterface(node));
            }
        }

        for (Node child = node.getFirstChild(); child != null;
             child = child.getNextSibling()) {
            getNodeInfo(child, xmlcDoc);
        }
    }

}