/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// Toolbox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.WarPlan;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.AbstractEchoBuilder;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.Writer;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.event.WriteListener;

// JDK
import java.awt.Component;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.StringTokenizer;
import java.util.ResourceBundle;
import java.util.Properties;

//
public class RunBuilder extends AbstractEchoBuilder {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    //
    private File          startServerDirectory = null;

    /**
     * Method declaration
     *
     *
     * @param in
     *
     * @return
     */
    public static File createPolicyFile(String path) {

        // strings not to be resourced
        final String POLICY_LINE1 = "grant {";            // nores
        final String POLICY_LINE2 =
            "permission java.security.AllPermission;";    // nores
        final String POLICY_LINE3 = "};";                 // nores
        File         in = new File(path);
        File         out = null;

        if (in.exists()) {
            in.delete();
            in = null;
        }
        try {
            out = new File(path);
            out.getParentFile().mkdirs();
            out.createNewFile();
            PrintWriter writer = new PrintWriter(new FileWriter(out));

            writer.println(POLICY_LINE1);
            writer.println(POLICY_LINE2);
            writer.println(POLICY_LINE3);
            writer.flush();
            writer.close();
            out = null;
            out = new File(path);
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        return out;
    }

    /**
     *
     * @param listeners
     */
    public RunBuilder(WriteListener listener) {
        super(listener);
    }

    /**
     * Make content, policy file, process templates, configure project for
     * running enhydra and create archive.
     */
    public void buildImpl() {
        PathHandle ph = null;

        if (getProject().isDeployRun()) {
            echo(new String());
            ph =
                PathHandle.createPathHandle(getProject().getDeployBootstrapPath());
            if (ph.isFile()) {

                // setup IDE for run
                if (isFresh()) {
                    if (!ToolBoxInfo.isEnhydra3()) {
                        refreshProgress(25,
                                        res.getString("Deploying_policy_file"));
                        echo(res.getString("Deploying_policy_file"));
                        makePolicy();
                    }
                }
                if (isFresh()) {
                    if (getProject().isDeployStartupJava()
                            || getProject().isDeployStartupRun()) {
                        refreshProgress(65,
                                        res.getString("Configuring_project"));
                        echo(res.getString("Configuring_project"));
                        makeStartup();
                    }
                }
            } else if (ph.isDirectory()) {

                // auto-deploy archive
                ph =
                    PathHandle.createPathHandle(getProject().getAutoDeployFilePath());
                if (ph.isEmpty()) {
                    echo("No archive to deploy");
                } else if (ph.getFile().isFile()) {
                    File out = null;

                    out = new File(getProject().getDeployBootstrapPath());
                    out = new File(out, ph.getFile().getName());
                    try {
                        FileUtil.copy(ph.getFile(), out);
                    } catch (ToolException e) {
                        echo(e);
                    }
                } else {
                    echo("Deployment archive not found: " + ph.getPath());
                }
            }
        }
        refreshProgress(100, new String());
    }

    //
    //
    //

    /**
     * Create policy file for security manager
     */
    private File createPolicy() {
        StringBuffer policyPath = new StringBuffer();
        File         policy = null;

        policyPath.append(PathUtil.getDeployConfPath(getProject()));
        policyPath.append(File.separator);
        policyPath.append(Constants.FILE_JAVA_POLICY);
        policy =
            RunBuilder.createPolicyFile(policyPath.toString().replace('\\',
                '/'));
        return policy;
    }

    /**
     * Copy or create policy file as needed.
     */
    private void makePolicy() {

        // Find all images in project and copy using
        // the mapping table.
        OtterTextFileNode node = getProject().getFirstPolicy();

        if (node == null) {
            File file = null;

            file = createPolicy();
            echo(res.getString("Creating_policy_file")
                 + PathHandle.createPathString(file));
        } else {
            File source = null;
            File dest = null;

            source = new File(node.getFilePath());
            dest = new File(PathUtil.getDeployConfPath(getProject()),
                            source.getName());
            try {
                FileUtil.copy(source, dest);
            } catch (ToolException e) {
                echo(e);
                e.printStackTrace();
            }
        }
    }

    /**
     * Configure project for starting server
     */
    private void makeStartup() {
        if (getProject().isDeployStartupJava()) {
            OtterNodeFactory     factory = null;
            StartServerGenerator start = null;
            PathHandle           startPath = null;

            factory = getProject().getNodeFactory();
            start = new StartServerGenerator(getProject());
            try {
                start.create();
                startPath = PathHandle.createPathHandle(start.getFile());
                factory.createJavaFileNode(getProject(), startPath.getPath());
                echo(res.getString("Creating_startup") + startPath.getPath());
            } catch (java.io.IOException e) {
                echo(e);
                e.printStackTrace();
            }
        }
        if (getProject().isDeployStartupRun()) {
            getProject().configureRunClass();
        }
    }

}
