/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.dods;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.swing.OutputPanel;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.DirectoryFilter;

// ToolBox
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;


// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.PrintWriter;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.beans.*;

// Standard imports
import java.awt.event.*;
import java.io.File;
import java.util.ResourceBundle;

import org.enhydra.dods.Common;
import org.enhydra.dods.util.XmlUtil;
import org.enhydra.dods.util.XmlUtilException;
import java.util.*;
import org.enhydra.dods.generator.DODSWizard;
import org.enhydra.dods.generator.DODSGenerator;

//
public class GeneralPanel extends JPanel implements Instructor, DodsStep {

    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private GridBagLayout       layoutMain;
    private JPanel              panelDomlFile;
    private GridBagLayout       layoutDomlFile;
    private JTextField          textDomlFile;
    private JButton             buttonDomlFile;
    private JPanel              panelRootPath;
    private GridBagLayout       layoutRootPath;
    private JTextField          textRootPath;
    private JButton             buttonRootPath;
    private JPanel              panelMessage;
    private JCheckBox           checkSqlGenerate;
    private JButton             buttonMessage;
    private JCheckBox           checkSqlSplitter;
    private JCheckBox           checkJavaGenerate;
    private JCheckBox           checkBuild;
    private JTabbedPane         tabMessage;
    private JPanel              panelMessageOption;
    private OutputPanel         outputPanel;
    private GridBagLayout       layoutMessageOption;
    private BorderLayout        layoutMessage;
    private CheckBoxListener    checkBoxListener = null;
    private boolean             ran = false;
    private JComboBox           comboType;
    private WeakReference       projectRef = null;
    private GridBagLayout       layoutType;
    private LocalButtonListener buttonListener = null;

    private JLabel jTemplateTypeLabel = new JLabel();
    private JCheckBox jCheckBoxDoml2Html = new JCheckBox();
    private JCheckBox jCheckBoxDoml2Pdf = new JCheckBox();
    private JCheckBox jCheckBoxDoml2Xmi = new JCheckBox();
    private JCheckBox jCheckBoxDoml2Ptl = new JCheckBox();
    private JPanel panelDocumentation;
    private JPanel panelTemplateSet;
    private GridBagLayout       layoutExtensions;

    private JLabel              labelRootPath;
    private JLabel              labelDomlFile;
    private JLabel              labelDocumentation;
    private JLabel              labelTemplateSet;
    private JComboBox jComboBoxTemplateSet = new JComboBox();
//    XYLayout xyLayout = new XYLayout();

//    private DODSGenerator generator;

    public GeneralPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "General";
    }

    // implements Instructor
    public String getTitle() {
        return "Dods generator";
    }

    // implements Instructor
    public String getInstructions() {
        return "Please select Doml file and choose action.";
    }

    // implements DodsStep
    public void refresh() {

        // nothing to refresh
    }

    // implements DodsStep
    public void clearAll() {
        outputPanel.clearAll();
        removeAll();
    }

    // implements DodsStep
    public void read(OtterProject project) {

//      if(project.getDodsRootPath()==null) {
        PathHandle path = null;
        int        type = -1;
        String projectRoot = project.getSourcePath();
        if(projectRoot.indexOf(";")!=-1)
          projectRoot = projectRoot.substring(0, projectRoot.indexOf(";"));
        path = PathHandle.createPathHandle(projectRoot);

        if (isValidDeployRoot(path)) {
            textRootPath.setText(path.getPath());
        } else {
            textRootPath.setText(new String());
        }
/*      } else {
            textRootPath.setText(project.getDodsRootPath());
      } */

        textRootPath.setToolTipText(textRootPath.getText());

        PathHandle domlPath = null;
        String[] ext = {"doml","DOML"};
        OtterFileNode[] domlFileNodes = project.findFileNodesByType(ext);
        String domlFileName= null;
        if(domlFileNodes.length > 0)
         {
            domlFileName = domlFileNodes[0].getFilePath();
         }
        if(domlFileName!=null)
        {
            domlPath = PathHandle.createPathHandle(domlFileName);
            if (domlPath.isFile()) {
              textDomlFile.setText(domlPath.getPath());
            } else {
              textDomlFile.setText(new String());
             }
          textDomlFile.setToolTipText(textDomlFile.getText());
        } else {
            textDomlFile.setText(project.getDomlFilePath());
        }

       //Dusan 24.01.2003.
        if (project.isOpenBuild()) {

            // Primetime
            checkBuild.setEnabled(true);
            checkBuild.setSelected(project.isDodsBuild());
        } else {

            // Win32 and JDeveloper
            checkBuild.setEnabled(false);
            checkBuild.setSelected(false);
        }

      }

    // implements DodsStep
    public void write(OtterProject project) throws DataValidationException {

       projectRef = new WeakReference(project);

       project.setDomlFilePath(textDomlFile.getText());

       project.setDodsRootPath(textRootPath.getText());


      //Dusan 24.01.2003.
        if (project.isOpenBuild()) {
            project.setDodsBuild(checkBuild.isSelected());
        } else {
            // Win32 and JDeveloper
            project.setDodsBuild(false);
        }

      //Dusan 15.3.2003.
      saveParameters(project);

  }

    void saveParameters(OtterProject proj) {

        OtterProject project = proj;

        project.setDodsParameters("dods:build_all");
        if (checkSqlGenerate.isSelected() && !checkSqlSplitter.isSelected() && !checkJavaGenerate.isSelected()) {
            project.setDodsParameters("dods:sql");
        }
        if (checkSqlGenerate.isSelected() && checkSqlSplitter.isSelected() && !checkJavaGenerate.isSelected()) {
            project.setDodsParameters("dods:sqlsplit");
        }
        if (!checkSqlGenerate.isSelected() && checkJavaGenerate.isSelected()) {
            project.setDodsParameters("dods:javaNoCompile");
        }
        if (checkSqlGenerate.isSelected() && checkJavaGenerate.isSelected()) {
            if (!checkSqlSplitter.isSelected())
              project.setDodsParameters("dods:noCompile");
        else
            project.setDodsParameters("dods:noCompileSplit");
        }

        if (jCheckBoxDoml2Html.isSelected())
            project.setDocumentationHtml("html");
        else
            project.setDocumentationHtml("");

        if (jCheckBoxDoml2Pdf.isSelected())
            project.setDocumentationPdf("pdf");
        else
            project.setDocumentationPdf("");

        if (jCheckBoxDoml2Xmi.isSelected())
            project.setDocumentationXmi("xmi");
        else
            project.setDocumentationXmi("");

        if (jCheckBoxDoml2Ptl.isSelected())
            project.setDocumentationPtl("ptl");
        else
            project.setDocumentationPtl("");

        project.setTemplateSet(((String)jComboBoxTemplateSet.getSelectedItem()).substring(1));

    }

    //
    protected WriteListener getWriteListener() {
        return outputPanel;
    }

    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[9];

        comps[0] = buttonDomlFile;
        comps[1] = jCheckBoxDoml2Html;
        comps[2] = jCheckBoxDoml2Pdf;
        comps[3] = jCheckBoxDoml2Xmi;
        comps[4] = jCheckBoxDoml2Ptl;
        comps[5] = checkSqlGenerate;
        comps[6] = checkSqlSplitter;
        comps[7] = checkJavaGenerate;
        comps[8] = checkBuild;
        return comps;
    }

    //
    private boolean isValidDodsRoot(PathHandle ph) {
        boolean valid = false;

        if (!ph.isFile()) {
            if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    private boolean isValidLogFile(PathHandle ph) {
        boolean valid = false;

        if (ph.hasExtension("txt")) {
            if (ph.isFile()) {
                valid = true;
            } else if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    //
    private void chooseDodsRoot() {
        File       choice = null;
        PathHandle path = null;

        choice =
            SwingUtil.getDirectoryChoice(this, textDomlFile.getText(),
                                         "Select Dods root directory");
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory()) {
            textDomlFile.setText(path.getPath());
            textDomlFile.setToolTipText(textDomlFile.getText());
        }
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonRootPath.addActionListener(buttonListener);
        buttonDomlFile.addActionListener(buttonListener);
        buttonMessage.addActionListener(buttonListener);

        checkBoxListener = new CheckBoxListener();
        checkSqlGenerate.addActionListener(checkBoxListener);
        checkJavaGenerate.addActionListener(checkBoxListener);
        checkBuild.addActionListener(checkBoxListener);
        //Dusan 12.02.2003.
        jCheckBoxDoml2Html.addActionListener(checkBoxListener);
        jCheckBoxDoml2Pdf.addActionListener(checkBoxListener);
        jCheckBoxDoml2Xmi.addActionListener(checkBoxListener);
        jCheckBoxDoml2Ptl.addActionListener(checkBoxListener);
   }

    private void jbInit() throws Exception {
        panelRootPath =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutRootPath =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textRootPath =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonRootPath =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        labelRootPath = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                              JLabel.class.getName());
        labelDomlFile = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                              JLabel.class.getName());
        panelDomlFile =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutDomlFile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textDomlFile =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonDomlFile =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelMessage = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        //Dusan 12.02.2003.
        panelDocumentation =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        labelDocumentation = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                              JLabel.class.getName());
        //Dusan 12.3.2003.
        panelTemplateSet  =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        labelTemplateSet = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                              JLabel.class.getName());
        layoutExtensions =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        jCheckBoxDoml2Html =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        jCheckBoxDoml2Pdf =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        jCheckBoxDoml2Xmi =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        jCheckBoxDoml2Ptl =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());

        checkSqlGenerate =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkSqlSplitter =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkJavaGenerate =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkBuild =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        buttonMessage =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tabMessage =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        panelMessageOption =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        layoutMessageOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMessage =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        comboType = (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JComboBox.class.getName());
        layoutType =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());

        DODSGenerator generator = new DODSGenerator();

        HashSet dirs = Common.getAllTemplateSets();
        String item= null;
        int i = 0;
        int selIndex = 0;
        for (Iterator iter = dirs.iterator(); iter.hasNext(); i++) {
          System.out.println("41: "+i);
            item = (String)iter.next();
            System.out.println("42: "+item);
            jComboBoxTemplateSet.addItem(" "+item);
            if (item.equals(generator.getTemplateSet()) )
                selIndex = i;
        }
        jComboBoxTemplateSet.setSelectedIndex(selIndex);
        System.out.println(51);
        jComboBoxTemplateSet.setMinimumSize(new Dimension(121, 20));
        System.out.println(52);

//        checkBuild = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
        textRootPath.setEnabled(false);
    textRootPath.setMinimumSize(new Dimension(200, 21));
    textRootPath.setPreferredSize(new Dimension(200, 21));
        textRootPath.setEditable(false);
        textRootPath.setText("Output directory");
        buttonRootPath.setText("Set...");
    buttonRootPath.setMaximumSize(new Dimension(61, 24));
    buttonRootPath.setMinimumSize(new Dimension(61, 24));
    buttonRootPath.setToolTipText("");
//    buttonRootPath.setMargin(new Insets(3, 130, 15, 65));
    buttonRootPath.setMnemonic('0');
    buttonRootPath.setText("Set...");

    //        panelRootPath.setLayout(xyLayout);
    panelRootPath.setLayout(layoutDomlFile);
    panelRootPath.setAlignmentX((float) 0.1);
    panelRootPath.setAlignmentY((float) 0.1);
/*    panelRootPath.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Output directory"));
    panelRootPath.setMinimumSize(new Dimension(97, 45)); */
    panelRootPath.setMinimumSize(new Dimension(80, 45));

    labelRootPath.setHorizontalAlignment(SwingConstants.RIGHT);
    labelRootPath.setText("Output directory:");
        textDomlFile.setEnabled(true);
    textDomlFile.setMinimumSize(new Dimension(200, 21));
    textDomlFile.setPreferredSize(new Dimension(200, 21));
        textDomlFile.setEditable(true);
        textDomlFile.setText("");
        buttonDomlFile.setMaximumSize(new Dimension(61, 24));
    buttonDomlFile.setMinimumSize(new Dimension(61, 24));
    buttonDomlFile.setPreferredSize(new Dimension(61, 24));
    buttonDomlFile.setText("Set...");
        buttonMessage.setText("...");
        panelDomlFile.setLayout(layoutDomlFile);
/*        panelDomlFile.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Doml file")); */
    panelDomlFile.setMinimumSize(new Dimension(97, 45));
    labelDomlFile.setHorizontalAlignment(SwingConstants.RIGHT);
    labelDomlFile.setText("Doml file:");

        //Dusan 12.02.2004.
        panelDocumentation.setLayout(layoutExtensions);
/*        panelDocumentation.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Documentation:")); */
    panelDocumentation.setMinimumSize(new Dimension(375, 45));
    labelDocumentation.setHorizontalAlignment(SwingConstants.RIGHT);
    labelDocumentation.setText("Documentation:");
        panelTemplateSet.setLayout(layoutExtensions);
/*        panelTemplateSet.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Template set:")); */
    labelTemplateSet.setHorizontalAlignment(SwingConstants.RIGHT);
    labelTemplateSet.setText("Template set:");
//        jTemplateTypeLabel.setText("Extensions :");
        jCheckBoxDoml2Html.setText("HTML");
        jCheckBoxDoml2Html.setSelected(false);
        jCheckBoxDoml2Pdf.setText("PDF");
        jCheckBoxDoml2Pdf.setSelected(false);
        jCheckBoxDoml2Xmi.setText("XMI");
        jCheckBoxDoml2Xmi.setSelected(false);
        jCheckBoxDoml2Ptl.setText("PTL");
        jCheckBoxDoml2Ptl.setSelected(false);

        checkSqlGenerate.setText("SQL generate");
        checkSqlGenerate.setSelected(true);
        checkSqlSplitter.setText("SQL Splitter");
        checkSqlSplitter.setSelected(true);
        checkJavaGenerate.setText("Java generate");
        checkJavaGenerate.setSelected(true);
        checkBuild.setText("Invoke Dods Generator during Project Make/Rebuild");
        checkBuild.setSelected(false);
        panelMessageOption.setLayout(layoutMessageOption);
        this.setAlignmentX((float) 0.1);
    this.setAlignmentY((float) 0.1);
    this.setPreferredSize(new Dimension(500, 670));
    panelMessage.setMinimumSize(new Dimension(41, 70));

//    panelRootPath.add(labelRootPath, new XYConstraints(1, 1, -1, -1));
//    panelRootPath.add(textRootPath, new XYConstraints(130, 1, 130, 50));

        panelRootPath.add(labelRootPath,
                            new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0,
                                                   GridBagConstraints.WEST,
                                                   GridBagConstraints.HORIZONTAL,
//                                                   new Insets(3, 5, 3, 5), 0,
//                                                   0));
                                                   new Insets(1, 1, 1, 1), 0,
                                                   0));
    panelRootPath.add(textRootPath,
                            new GridBagConstraints(1, 0, 1, 1, 0.8, 0.0,
                                                   GridBagConstraints.WEST,
                                                   GridBagConstraints.HORIZONTAL,
//                                                   new Insets(3, 5, 3, 5), 0,
//                                                   0));
                                                   new Insets(1, 1, 1, 1), 0,
                                                   0));
        panelRootPath.add(buttonRootPath,
                            new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.NONE,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));

        System.out.println(8);
        panelDomlFile.add(labelDomlFile,
                              new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
        panelDomlFile.add(textDomlFile,
                                    new GridBagConstraints(1, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 0, 0));
        panelDomlFile.add(buttonDomlFile,
                            new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.NONE,
                                                   new Insets(3, 5, 3, 5), 0,
                                                   0));

        System.out.println(9);
        panelDocumentation.add(labelDocumentation,
                             new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));
        panelDocumentation.add(jCheckBoxDoml2Html,
                             new GridBagConstraints(1, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));
        panelDocumentation.add(jCheckBoxDoml2Pdf,
                             new GridBagConstraints(2, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));
        panelDocumentation.add(jCheckBoxDoml2Xmi,
                             new GridBagConstraints(3, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));
        panelDocumentation.add(jCheckBoxDoml2Ptl,
                             new GridBagConstraints(4, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));

        panelTemplateSet.add(labelTemplateSet,
                             new GridBagConstraints(0, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));
        panelTemplateSet.add(jComboBoxTemplateSet,
                             new GridBagConstraints(1, 0, 1, 1, 0.8, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 5, 3, 5), 0, 0));

        panelMessage.setLayout(layoutMessage);
/*        panelMessage.setBorder(BorderFactory.createTitledBorder(new EtchedBorder(),
                "Dods messages")); */

        this.setLayout(layoutMain);
        this.add(panelDomlFile,   new GridBagConstraints(0, 1, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 5), 0, 0));
        this.add(checkSqlGenerate,
                     new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0));
        this.add(panelDocumentation,
                         new GridBagConstraints(0, 3, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 5), 0, 0));
        this.add(panelTemplateSet,
                         new GridBagConstraints(0, 4, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 2, 2, 5), 0, 0));
        this.add(checkSqlSplitter,
                     new GridBagConstraints(1, 5, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0));
        this.add(checkJavaGenerate,
                      new GridBagConstraints(0, 6, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 10, 2, 10), 0, 0));
    this.add(checkBuild,    new GridBagConstraints(1, 6, 2, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0));
    this.add(panelMessage,         new GridBagConstraints(0, 2, 2, 1, 0.1, 0.0
            ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL, new Insets(-2, 8, 0, 2), 0, 60));
    panelMessage.add(tabMessage, BorderLayout.CENTER);
    this.add(panelRootPath,       new GridBagConstraints(0, 0, 2, 1, 0.1, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(-5, 3, 0, 7), 0, 0));
    System.out.println(11);
    tabMessage.add(outputPanel, "Messages");
    }

    private class CheckBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == checkSqlGenerate) {
         if (checkSqlGenerate.isSelected()) {
           checkSqlSplitter.setSelected(true);
           checkSqlSplitter.setEnabled(true);
           }
         else {
        checkSqlSplitter.setSelected(false);
        checkSqlSplitter.setEnabled(false);
           }
            }
            //Dusan 25.01.2003.
            if (source == checkSqlGenerate) {
         if (!checkSqlGenerate.isSelected()) {
           checkJavaGenerate.setSelected(true);
           }
            }
            if (source == checkJavaGenerate) {
         if (!checkJavaGenerate.isSelected()) {
           checkSqlGenerate.setSelected(true);
           checkSqlSplitter.setSelected(true);
           checkSqlSplitter.setEnabled(true);
           }
            }
      }
    }

    //
    private void chooseDomlFile() {
        File       choice = null;
        PathHandle path = null;
        ExtensionFilter filter = null;

        filter = new ExtensionFilter();
        filter.addExtension("doml");

/*        choice =
            SwingUtil.getFileChoice(this, textDomlFile.getText(),
              filter, "Select Doml file"); */
        choice =
            SwingUtil.getFileChoice(this, "c:\temp",
              filter, "Select Doml file");
        path = PathHandle.createPathHandle(choice);
        if (path.isFile()) {
            textDomlFile.setText(path.getPath());
            textDomlFile.setToolTipText(textDomlFile.getText());
        }
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonDomlFile) {
//Dusan 23.11.2002.
//                chooseDomlFile();
                browseForFile();
            } else if(source == buttonRootPath) {
                browseForDirectory();
            }
        }

    }

    /**
     * Method declaration
     *
     */
    private void browseForFile() {
        JFileChooser    chooser;
        ExtensionFilter filter;
        File            file;
        String          fileDir = new String();
        String          fileName = new String();

        filter = new ExtensionFilter();
        filter.addExtension(Constants.TYPE_DODS);
        filter.setDescriptionTitle(res.getString("filter_Doml_DescriptionTitle"));
        chooser = new JFileChooser();
        if (textDomlFile.getText().trim().length() > 0) {
            chooser.setCurrentDirectory(new File(textDomlFile.getText()));
        } else {
            PathHandle ph = null;
            chooser.setCurrentDirectory(new File(""));
        }
        chooser.setFileFilter(filter);
        chooser.setDialogTitle(res.getString("chooser_DODS_DialogTitle"));
        chooser.setApproveButtonText(res.getString("OK"));
        int v = chooser.showOpenDialog(this);

        this.requestFocus();
        buttonDomlFile.requestFocus();
        if (v == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile() == null
                    || (!chooser.getSelectedFile().isFile())) {
                textDomlFile.setText(new String());
            } else {
                fileDir = chooser.getCurrentDirectory().toString();
                fileDir = stripSlash(fileDir);
                fileName = chooser.getSelectedFile().getName();
                String strDomlFile = fileDir + File.separator + fileName;
                textDomlFile.setText(strDomlFile.replace('\\','/'));
           }
        }
        chooser.removeAll();
        chooser = null;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @return
     */
    private String stripSlash(String dir) {
        String stripped = new String(dir);

        if (dir != null) {
            if (dir.length() > 0) {
                if (dir.endsWith(File.separator)) {
                    stripped = dir.substring(0, dir.length() - 1);
                }
            }
        }
        return stripped;
    }

    //
    private boolean isValidDeployRoot(PathHandle ph) {
        boolean valid = false;

        if (!ph.isFile()) {
            if (ph.getParent() != null) {
                valid = ph.getParent().isDirectory();
            }
        }
        return valid;
    }

    /**
     * Method declaration
     *
     */
    private void browseForDirectory() {
        JFileChooser    chooser;
        File            file;
        String          fileDir = new String();
        String          fileName = new String();

        chooser = new JFileChooser();
        if (textRootPath.getText().trim().length() > 0) {
            chooser.setCurrentDirectory(new File(textRootPath.getText()));
        } else {
            PathHandle ph = null;

            chooser.setCurrentDirectory(new File(""));
        }
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setFileFilter((javax.swing.filechooser.FileFilter) new DirectoryFilter());
        chooser.setDialogTitle(res.getString("chooser_RootPath_DialogTitle"));
        chooser.setApproveButtonText(res.getString("OK"));
        int v = chooser.showOpenDialog(this);

        this.requestFocus();
        buttonRootPath.requestFocus();
        if (v == JFileChooser.APPROVE_OPTION) {
            if (chooser.getSelectedFile() == null) {
                textRootPath.setText(new String());
            } else {
                fileDir = chooser.getCurrentDirectory().toString();
                fileDir = stripSlash(fileDir);
                fileName = chooser.getSelectedFile().getName();
                String strRootPath = fileDir + File.separator + fileName;
                textRootPath.setText(strRootPath.replace('\\','/'));
            }
        }
        chooser.removeAll();
        chooser = null;
    }

/*    public void setDODSGenerator(DODSGenerator gen) {
      generator = gen;
    } */

}
