/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// Kelp imports
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;

// standard imports
import java.awt.*;
import javax.swing.*;
import java.beans.*;
import java.util.ResourceBundle;
public class XMLCOptionTab extends JPanel {
    static ResourceBundle   res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private BorderLayout    borderLayout1 = new BorderLayout();
    private JTabbedPane     tab;
    private OtterNode       node = null;
    private XMLCOptionPanel filePanel;
    private XMLCOptionPanel packagePanel;
    private XMLCOptionPanel projectPanel;

    public XMLCOptionTab() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void setNode(OtterNode n) {
        node = n;
    }

    public OtterNode getNode() {
        return node;
    }

    public void readProperties() {
        filePanel.readProperties();
        packagePanel.readProperties();
        projectPanel.readProperties();
    }

    public void writeProperties() {
        filePanel.writeProperties();
        packagePanel.writeProperties();
        projectPanel.writeProperties();
    }

    public void setEnabled(boolean b) {
        super.setEnabled(b);
        filePanel.setEnabled(b);
        packagePanel.setEnabled(b);
        projectPanel.setEnabled(true);
    }

    public void init() {
        OtterNode parent = null;

        if (node != null) {
            projectPanel.setNode(node.getProject());
            if (node instanceof OtterXMLCNode) {
                filePanel.setNode(node);
                parent = node.getParent();
                if (parent == null || parent instanceof OtterProject) {
                    tab.remove(packagePanel);
                } else {
                    packagePanel.setNode(node.getParent());
                }
            } else {
                packagePanel.setNode(node);
                tab.remove(filePanel);
            }
        }
    }

    private void pmInit() {
        setBorder(filePanel.getBorder());
        filePanel.setBorder(null);
        packagePanel.setBorder(null);
        projectPanel.setBorder(null);
    }

    private void jbInit() throws Exception {
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        filePanel =
            (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOptionPanel.class.getName());
        packagePanel =
            (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOptionPanel.class.getName());
        projectPanel =
            (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOptionPanel.class.getName());
        this.setLayout(borderLayout1);
        this.add(tab, BorderLayout.CENTER);
        tab.add(filePanel, res.getString("File"));
        tab.add(packagePanel, res.getString("Package_Folder"));
        tab.add(projectPanel, res.getString("Project"));
    }

}
