/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */
package org.enhydra.kelp.forte.actions;

// OpenIDE
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.openide.DialogDescriptor;
import org.openide.TopManager;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;
import org.openide.cookies.ViewCookie;
import org.openide.filesystems.Repository;
import org.openide.filesystems.JarFileSystem;
import org.openide.util.actions.SystemAction;
import org.openide.util.actions.CallableSystemAction;

// ToolBox
import org.enhydra.tool.codegen.Generator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.internal.Enhydra3AppGenerator;
import org.enhydra.tool.codegen.internal.WebAppGenerator;
import org.enhydra.tool.codegen.internal.ServiceGenerator;
import org.enhydra.tool.codegen.wizard.CodeGenWizard;
import org.enhydra.tool.codegen.wizard.GenSelectionEvent;
import org.enhydra.tool.codegen.wizard.GenSelectionListener;
import org.enhydra.tool.common.wizard.TBWizardButtonPanel;
import org.enhydra.tool.common.event.HelpListener;
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.ToolBoxInfo;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.codegen.CodeGenException;
import org.enhydra.kelp.common.codegen.ProjectGenUtil;
import org.enhydra.kelp.common.codegen.Enhydra3AppUtil;
import org.enhydra.kelp.common.codegen.ServiceUtil;
import org.enhydra.kelp.common.codegen.WebAppUtil;

// AddinForte
import org.enhydra.kelp.forte.node.ForteProject;
import org.enhydra.kelp.forte.node.XMLCDataObject;
import org.enhydra.kelp.forte.XMLCSettings;

// JDK
import java.awt.Dialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.util.ArrayList;
import java.io.IOException;
import java.beans.PropertyVetoException;
import javax.swing.JOptionPane;
import java.util.ResourceBundle;

/**
 * Action that can always be invoked and work procedurally.
 *
 * @author  rees0234
 */
public class AppWizardAction extends CallableSystemAction
    implements HelpListener, GenSelectionListener {

    private static String  anchor = "Idunno";
    private ProjectGenUtil util = null;

    public static final int NOAPP = -1;
    public static final int WEBAPP = 0;
    public static final int SERVICE = 1;
    public static final int ENHYDRA = 2;

    protected static int apptype = NOAPP;

    public static int getAppType()
    {
        return apptype;
    }

    public void performAction() {
        String[] options = TBWizardButtonPanel.getOptions();

        try {
            CodeGenWizard    wizard = new CodeGenWizard();
            wizard.getInnerPanel().initPreferredSize();
            DialogDescriptor dscr =
                new DialogDescriptor(wizard.getInnerPanel(),
                                     wizard.getTitle(), true, options,
                                     options[1],
                                     DialogDescriptor.BOTTOM_ALIGN,
                                     HelpCtx.DEFAULT_HELP,
                                     wizard.createButtonListener());
            Dialog           dlg = TopManager.getDefault().createDialog(dscr);

            wizard.addSelectionListener(this);
            wizard.addHelpListener(this);
            wizard.getInnerPanel().preShow();
            wizard.setAllowRootEdit(false);
            dlg.show();
            if (util == null) {
              // cancelled prior to selection
            } else if (wizard.getOption() != JOptionPane.CANCEL_OPTION)

            {
                ForteProject project = new ForteProject();
               // FileSystem system = project.getRootFileSystem();
                FileObject readme = null;
                FileObject rootFile = project.getRootFolder();
                String src = XMLCSettings.getDefault().getRoot();
              //refresh all the project dir so we can find the generated files:
                rootFile.refresh();
                try {
                    rootFile.getFileObject(src);
                }  catch (IllegalArgumentException e) {
                    System.err.println(e);
                    return;
                }
                if (rootFile.getChildren().length <= 0)
                    return;

                XMLCSettings.getDefault().setSource(org.enhydra.kelp.common.Constants.DIR_SRC);

                util.importFiles(wizard.getGeneratedFiles());
                //add enhydra stuff to classpath:
                Repository rep = TopManager.getDefault().getRepository();
                String classpath = project.getClassPath();
                if (apptype == WEBAPP || apptype == ENHYDRA)
                {

                    String target = null;
                    StringBuffer buf = new StringBuffer();
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    String base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("enhydra.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar).append("build").append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("toolbox.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("xmlc.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("xerces.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("tomcat.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("core.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("util.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("gnu-regexp.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                                        buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                      append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("wireless.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("lib").
                    append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("xhtml.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
//                    if (! project.cpContains(target))
                    {
                        JarFileSystem jar = new JarFileSystem();
                        jar.setJarFile(new File(target));
                        rep.addFileSystem(jar);
                    }
//Dusan 24.11.2002.
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("dods").
                    append(File.separatorChar).append("lib").append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("dods.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
                    {
                      JarFileSystem jar = new JarFileSystem();
                      jar.setJarFile(new File(target));
                      rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).append("dods").
                    append(File.separatorChar).append("lib").append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("dods-runtime.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());

                    {
                      JarFileSystem jar = new JarFileSystem();
                      jar.setJarFile(new File(target));
                      rep.addFileSystem(jar);
                    }
                    buf.append(ToolBoxInfo.getEnhydraRoot()).append(File.separatorChar).
                    append("lib").append(File.separatorChar);
                    base = buf.toString();
                    buf.delete(0, buf.length());
                    buf.append(base).append("log4j.jar");
                    target = buf.toString();
                    buf.delete(0, buf.length());
                    {
                      JarFileSystem jar = new JarFileSystem();
                      jar.setJarFile(new File(target));
                      rep.addFileSystem(jar);
                    }
                  }
                readme = rootFile.getFileObject("readme", "html");
                XMLCDataObject target = null;
                          //open readme.html:
                try {
                        target = (XMLCDataObject)DataObject.find(readme);
                 } catch (DataObjectNotFoundException e) {
                      System.err.println(e);
                 }
                 if (target != null)
                 {
                   ViewCookie viewit = (ViewCookie)target.getCookie(ViewCookie.class);
                   if (viewit != null)
                        viewit.view();
                 }
            }

        } catch (GeneratorException e) {
            e.printStackTrace(System.err);
        } catch (CodeGenException e) {
            e.printStackTrace(System.err);
        } catch (IOException e) {
            System.err.println(e);
        } catch (PropertyVetoException e) {
            System.err.println(e);
        }
        //force menu's enabled as this is not happening "naturally":
        ((XMLCAction)SystemAction.get(XMLCAction.class)).setEnabled(true);
        ((DeploymentAction)SystemAction.get(DeploymentAction.class)).setEnabled(true);

    }

    public String getName() {
        return CodeGenWizard.getDefaultTitle();
    }

    public boolean isEnabled()
    {
        return (new ForteProject().getRootFolder() != null);
    }

    protected String iconResource() {
        return "smallicon.gif";
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;

        // If you will provide context help then use:
        // return new HelpCtx (AppWizardAction.class);
    }

    /**
     * Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
     */
    protected void initialize() {
        super.initialize();
        putProperty(javax.swing.Action.SHORT_DESCRIPTION,
                    NbBundle.getMessage(AppWizardAction.class,
                                        "HINT_AppWiz_Action"));
    }

    // implements GenSelectionListener
    public void onSelection(GenSelectionEvent event) {
        ForteProject  project = new ForteProject();
        Generator     gen = event.getGenerator();
        CodeGenWizard wizard = event.getWizard();
        try {
            if (gen instanceof WebAppGenerator) {
                util = new WebAppUtil(project, (WebAppGenerator) gen);
                apptype = WEBAPP;
            } else if (gen instanceof ServiceGenerator) {
                util = new ServiceUtil(project, (ServiceGenerator) gen);
                apptype = SERVICE;
            } else if (gen instanceof Enhydra3AppGenerator) {
                util = new Enhydra3AppUtil(project,
                                           (Enhydra3AppGenerator) gen);
                apptype = ENHYDRA;
            }
            if (getAddinSteps().length > 0)
                util.getGenerator().setAddinSteps(getAddinSteps());
            String javaPath = System.getProperty("java.home");
            //assume parent is root jdk dir as java.home points to jre:
            if (javaPath.endsWith(File.separatorChar + "jre"))
                javaPath = javaPath.substring(0, javaPath.length() - 4);
            else
            {
                StringBuffer buf = new StringBuffer(javaPath);
                while (buf.charAt(buf.length() - 1) != File.separatorChar)
                {
                    buf.deleteCharAt(buf.length() - 1);
                }
                buf.deleteCharAt(buf.length() - 1);
                javaPath = buf.toString();
            }
            System.out.println("java path = " + javaPath);
            util.getGenerator().setJavaPath(javaPath);

            wizard.setCodeGen(util.getCodeGen());
        } catch (CodeGenException e) {
            e.printStackTrace(System.err);
        }
    }

    // implements HelpListener
    public void onHelp(HelpEvent event) {
        URL          url = null;
        String[]     home = new String[2];
        home[0] = System.getProperty("netbeans.home");
        home[1] = System.getProperty("netbeans.user");
        StringBuffer buf = new StringBuffer();
        buf.append(KelpInfo.getAddinHelpURL(home));
        buf.append('#').append(anchor);
        try {
          url = new URL(buf.toString());
          TopManager.getDefault().showUrl(url);
        } catch (MalformedURLException e) {
          e.printStackTrace(System.err);
        }
    }

    // Steps added to generated readme.html file.
    private String[] getAddinSteps() {
        ResourceBundle bundle = NbBundle.getBundle(AppWizardAction.class);
        ArrayList list = new ArrayList();
        String[] steps = new String[0];

        if (apptype == WEBAPP)
        {
            list.add(bundle.getString("readmeWeb1"));
            list.add(bundle.getString("readmeWeb2"));
            list.add(bundle.getString("readmeWeb3"));
            list.add(bundle.getString("readmeWeb4"));
        }
        else if (apptype == SERVICE)
        {
            return steps;
        }

        else
        {
            list.add(bundle.getString("readmeEnApp1"));
            list.add(bundle.getString("readmeEnApp2"));
            list.add(bundle.getString("readmeEnApp3"));
            list.add(bundle.getString("readmeEnApp4"));
        }

        list.trimToSize();

        steps = new String[list.size()];
        steps = (String[]) list.toArray(steps);
        list.clear();
        return steps;
    }


}
