/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte;

import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.PropUtil;

import java.util.Hashtable;
import java.io.*;
/** Options for something or other.
 *
 * @author  rees0234
 */
public class DodsSettings extends SystemOption implements PropertyKeys{

    static final long serialVersionUID = 8806955980938332585L;

    protected Hashtable nodeProps = null;

    public String displayName () {
        return NbBundle.getMessage (DodsSettings.class, "LBL_deploy_settings");
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you provide context help then use:
        // return new HelpCtx (DeploySettings.class);
    }

    /** Default instance of this system option, for the convenience of associated classes. */
    public static DodsSettings getDefault ()
    {
        return (DodsSettings) findObject (DodsSettings.class, true);
    }

   public void writeExternal (ObjectOutput out) throws IOException {
        out.writeObject(propertyValue("DodsParameters"));
        out.writeObject(propertyValue("DomlFilepath"));
        out.writeObject(propertyValue("enhydra.dods.java.project.path"));
        out.writeObject(propertyValue("DodsEnhydraPath"));
    }

   public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        setProperty("DodsParameters", (String)in.readObject());
        setProperty("DomlFilepath", (String)in.readObject());
        setProperty("enhydra.dods.java.project.path", (String)in.readObject());
        setProperty("DodsEnhydraPath", (String)in.readObject());

    }

    public void setProperty(String key, String value)
    {
        putProperty(key, value, true);
    }

    public String propertyValue(String key)
    {
        return (String)getProperty(key);
    }

    public void setNodeProperty(String key, String prop)
    {
        if (nodeProps == null)
            nodeProps = new Hashtable();

        if (prop != null)
            try {
                nodeProps.put(key, prop);
            }catch (NullPointerException e) {
                System.err.println(e);
            }
        else
            nodeProps.remove(key);
    }

    public String getNodeProperty(String key)
    {
        if (nodeProps != null)
        {
            String res = (String)nodeProps.get(key);
            return res;
        }

        return null;
    }

    public String getDeployRoot()
    {
        return propertyValue(DEPLOY_PATH_ROOT);
    }

    public void setDeployRoot(String root)
    {
        setProperty(DEPLOY_PATH_ROOT, root);
    }

    public boolean isGlobal()
    {
        return false;
    }
}


