/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

import java.util.Enumeration;
import java.io.*;

/** Recognizes some set of files in the Repository as belonging together.
 *
 * @author rees0234
 */
public class DodsDataLoader extends MultiFileLoader {

    static final long serialVersionUID = 7377840573359349120L;

    private static final String TEMP_EXT = "in";
//    private static final String CONF_EXT = "conf";

    public static final String PROP_DODS_EXTENSIONS = "dods.extensions";

    public DodsDataLoader ()
{
    this (DodsDataObject.class);
    }

    public DodsDataLoader (Class recognizedObject)
{
        super (recognizedObject);
    }

    protected void initialize () {

        setDisplayName (NbBundle.getMessage(DodsDataLoader.class, "LBL_Dodsloader"));

        setActions (new SystemAction[] {
//                        SystemAction.get (OpenAction.class),
                        null,
//                        SystemAction.get (FileSystemAction.class),
                        null,
                        null,
                        /*
                        SystemAction.get (CompileAction.class),
                        null,
                        SystemAction.get (ExecuteAction.class),
                        null,
                        */
//                        SystemAction.get (CutAction.class),
                        null,
//                        SystemAction.get (CopyAction.class),
                        null,
//                        SystemAction.get (PasteAction.class),
                        null,
                        null,
//                        SystemAction.get (DeleteAction.class),
                        null,
//                        SystemAction.get (RenameAction.class),
                        null,
                        null,
//                        SystemAction.get (SaveAsTemplateAction.class),
                        null,
//                        SystemAction.get (ToolsAction.class),
//                        SystemAction.get (PropertiesAction.class),
                        null
                    });

//        super.initialize();

    }

    protected FileObject findPrimaryFile (FileObject fo) {
        // This can of course be much more sophisticated.
        Enumeration extSet = getExtensions().extensions();
        while (extSet.hasMoreElements())
        {
            String ext = (String)extSet.nextElement();
            if (fo.hasExt (ext))
                return fo;
        }
        return null;
    }

    protected MultiDataObject createMultiObject (FileObject primaryFile)
    throws DataObjectExistsException, IOException {
        return new DodsDataObject (primaryFile, this);
    }

    protected MultiDataObject.Entry createPrimaryEntry (MultiDataObject obj, FileObject primaryFile) {
        // Entry for the important file: by default, is preserved during all operations.
        return new FileEntry (obj, primaryFile);
        // return new DodsFormat (obj, primaryFile);
    }

    protected MultiDataObject.Entry createSecondaryEntry (MultiDataObject obj, FileObject secondaryFile) {
        secondaryFile.setImportant (false);
        return new FileEntry.Numb (obj, secondaryFile); // discard whenever needed
    }

    /** @return The list of extensions this loader recognizes
    * (the default contains only java)
    */
    public ExtensionList getExtensions () {
	ExtensionList extensions = (ExtensionList)getProperty(PROP_DODS_EXTENSIONS);
	if (extensions == null) {
            extensions = new ExtensionList();
     //these should be read from global props instead of hard coded   ?
            extensions.addExtension(TEMP_EXT);
//            extensions.addExtension(CONF_EXT);

	    putProperty(PROP_DODS_EXTENSIONS, extensions, false);
        }
        return extensions;
    }

    /** Sets the extension list for this data loader.
    * @param ext new list of extensions.
    */
    public void setExtensions(ExtensionList ext) {
        putProperty(PROP_DODS_EXTENSIONS, ext, true);
    }

}
