/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.services;

import java.io.IOException;
import java.util.*;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.filesystems.*;

/** Does the work of cleaning up after an external compiler.
 *
 * @author rees0234
 */
public class XMLCCleanCompilerGroup extends CompilerGroup {

    private Set compilers = new HashSet (); // Set<XMLCCompilerCleanCompiler>

    public void add (Compiler c) throws IllegalArgumentException {
        if (! (c instanceof XMLCCleanCompiler)) throw new IllegalArgumentException ();
        compilers.add (c);
    }

    public boolean start () {
        boolean ok = true;
        Iterator it = compilers.iterator ();
        while (it.hasNext ()) {
            XMLCCleanCompiler c = (XMLCCleanCompiler) it.next ();
            if (c.isUpToDate ()) continue;
            FileObject fo = c.getFileObject ();
            FileObject toKill = FileUtil.findBrother (fo, "compiledExt");
            if (toKill == null) continue;
            try {
                FileLock lock = toKill.lock ();
                try {
                    toKill.delete (lock);
                    fireProgressEvent (new ProgressEvent (this, fo, ProgressEvent.TASK_WRITING));
                } finally {
                    lock.releaseLock ();
                }
            } catch (IOException ioe) {
                fireErrorEvent (new ErrorEvent (this, toKill, 0, 0, ioe.toString (), null));
                ok = false;
            }
        }
        return ok;
    }

}
