/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.services;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.execution.NbProcessDescriptor;

public class XMLCCompilerGroup extends ExternalCompilerGroup {

    private String myOpt = null; // will be set after compilers are added

    public void add (Compiler c) throws IllegalArgumentException {
        // Calling super is necessary: that picks up the file list, etc.
        super.add (c);
        if (! (c instanceof XMLCCompiler)) throw new IllegalArgumentException ();
        XMLCCompiler comp = (XMLCCompiler) c;
        // This will be the same value for all added compilers (if more than one),
        // because of the correct implementation of compilerGroupKey:
        myOpt = comp.getMyOpt ();
    }

    protected Process createProcess (NbProcessDescriptor desc, String[] files) throws IOException {
        return desc.exec (new Format (files, myOpt));
    }

    public static class Format extends ExternalCompilerGroup.Format {

        public static final String TAG_MYOPTIONVAL = "myOpt";

        public Format (String[] files, String myOpt) {
            super (files);
            Map map = getMap ();
            map.put (TAG_MYOPTIONVAL, myOpt);
        }

    }

}
