/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.services;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.openide.compiler.ExternalCompilerType;
import org.openide.compiler.CompilerType;
import org.openide.compiler.Compiler;
import org.openide.compiler.*;
import org.openide.cookies.CompilerCookie;
import org.openide.execution.NbProcessDescriptor;
import org.openide.loaders.DataObject;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Runs an external process to "compile" a type of file.
 *
 * @author rees0234
 */
public class XMLCCompilerType extends ExternalCompilerType {

//these are all of our xmlc settings:
    static final long serialVersionUID = -2979099777437824015L;
//    private String workingDir;                                       // nores
//    private boolean autoPackage;                                   // nores
//    private String libs;    // nores
//    private String outDir;                                // nores
//    private String customName;                                   // nores
//    private String classNameType;                             // nores
//    private String optionFile;                               // nores
//    private String xmlcParameters;                                // nores
//    public final static String   NAME_SELECTED = "enhydra.selected";    // nores
//    private String staticResource;                                // nores

//    public final static String   VALUE_ENHYDRA = "Enhydra";       // nores

    // directory names
//    public final static String[] DEFAULT_CONTENT = {
//        "css", "gif", "jpg", "js", "jsp", "wbmp"
//    };                                                            // nores

    //
//    public final String          XMLC_DOC_TYPES =
//        "enhydra.xmlc.docTypes";                                  // nores
//    private boolean build;    // nores
//    private boolean printDOM;                                  // nores
//    private boolean printParse;                                // nores
//    private boolean printDoc;                                  // nores
//    private boolean verboseMode;                                   // nores
//    private boolean printAccessor;                             // nores
//    private String outputToFilename;                          // nores
//    private String mapFromFolder;                    // nores
//    private String mapToPackage;                     // nores
//    public final String          XMLC_MAP_PACKAGE_LENGTH =
 //       "enhydra.xmlc.map.package.length";                        // nores
//    private int mapScope;
    /*
    static final ExternalCompiler.ErrorExpression[] ERROR_EXPRS = {
	// See docs for this class for how to construct them.
        new ExternalCompiler.ErrorExpression (NbBundle.getMessage (XMLCCompilerCompilerType.class, "LBL_error_exp_1"),
					      "(some|reg|exp|here)",
					      1, 2, 3, 4),
	// maybe some more choices as well
    };
    */

    private static final NbProcessDescriptor DEFAULT = new NbProcessDescriptor (
                // PROCESS NAME:
                // This need not be Javac, of course:
                "{" + ExternalCompilerGroup.Format.TAG_JAVAHOME + "}{" + ExternalCompilerGroup.Format.TAG_SEPARATOR +
		"}bin{" + ExternalCompilerGroup.Format.TAG_SEPARATOR + "}javac",
                // LIST OF ARGUMENTS INCLUDING OPTIONS:
                // And you need not specify a classpath if that does not apply to your situation:
                "-cp {" + ExternalCompilerGroup.Format.TAG_REPOSITORY + "}" +
                "{" + ExternalCompilerGroup.Format.TAG_PATHSEPARATOR + "}" + "{" + ExternalCompilerGroup.Format.TAG_CLASSPATH + "}" +
                "{" + ExternalCompilerGroup.Format.TAG_PATHSEPARATOR + "}" + "{" + ExternalCompilerGroup.Format.TAG_LIBRARY + "}" +
                "{" + ExternalCompilerGroup.Format.TAG_PATHSEPARATOR + "}" + "{" + ExternalCompilerGroup.Format.TAG_BOOTCLASSPATH + "} " +
                // You can modify the default format however you like, and insert or remove tags like so:
              //  "-myoption {" + XMLCCompilerCompilerGroup.Format.TAG_MYOPTIONVAL + "} " +
                // List of file names will go here:
                "{" + ExternalCompilerGroup.Format.TAG_FILES + '}',
                // DESCRIPTION FOR USER OF HOW TO MODIFY THE ARGUMENTS:
                NbBundle.getMessage (XMLCCompilerType.class, "MSG_format_hint")
            );

    // The default value for an unconfigured compiler.
    private String valueOfMyOption = "someDefaultValue";

    public XMLCCompilerType()
{
        // Override the default process choice made in ExternalCompilerType:
        setExternalCompiler (DEFAULT);
        /*
        // Install a default error expression, out of the default list presented to the user:
        setErrorExpression (ERROR_EXPRS[0]);
        */
    }

    public String getValueOfMyOption () {
        return valueOfMyOption;
    }

    public synchronized void setValueOfMyOption (String nue) {
        String old = valueOfMyOption;
        valueOfMyOption = nue;
        firePropertyChange ("valueOfMyOption", old, nue);
    }

/*    public boolean getBuild()
    {
        return build;
    }

    public void setBuild(boolean build)
    {
        this.build = build;
    }

    public boolean getPrintDOM()
    {
        return printDOM;
    }

    public void setPrintDOM(boolean printDOM)
    {
        this.printDOM = printDOM;
    }*/

    /*
    public HelpCtx getHelpCtx () {
        return new HelpCtx (XMLCCompilerType.class);
    }
    */

    public void prepareJob (CompilerJob job, Class type, DataObject obj) {
        Compiler cleaner;
        if (type.equals (CompilerCookie.Clean.class) ||
                type.equals (CompilerCookie.Build.class)) {
                    cleaner = new XMLCCleanCompiler (obj.getPrimaryFile ());
            job.add (cleaner);
        } else {
            cleaner = null;
        }
        if (type.equals (CompilerCookie.Compile.class) ||
                type.equals (CompilerCookie.Build.class)) {
                    Compiler compiler = new XMLCCompiler
                                (false, obj.getPrimaryFile (), getExternalCompiler (), getErrorExpression (), getValueOfMyOption ());
            job.add (compiler);
            if (cleaner != null) compiler.dependsOn (cleaner);
        } else if (cleaner == null) {
            // do not do anything--unrecognized compilation type
        }
    }

}
