/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.properties;

// JBuilder imports
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.help.HelpTopic;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.properties.PropertyPage;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.properties.XMLCNodePropertyPanel;

// AddinJBuilder
import org.enhydra.kelp.jbuilder.node.PrimeDocumentNode;

// JDK
import java.awt.BorderLayout;
import java.lang.ref.WeakReference;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class XMLCNodePropertyPage extends PropertyPage {

    // string not to be resourced
    private final String          HELP_LINK = KelpInfo.getAddinHelpURL();    // nores

    //
    private XMLCNodePropertyPanel nodePanel = new XMLCNodePropertyPanel();
    private BorderLayout          layoutMain = new BorderLayout();
    private WeakReference         nodeRef = null;

    /**
     * Constructor declaration
     *
     * @param topic
     */
    public XMLCNodePropertyPage(Object topic) {
        if (topic instanceof TextFileNode) {
            OtterXMLCNode     xmlcNode = null;
            OtterDocumentNode docNode = null;
            TextFileNode      nativeText = (TextFileNode) topic;

            docNode = new PrimeDocumentNode(nativeText);
            xmlcNode = new OtterXMLCNode(docNode);
            nodeRef = new WeakReference(xmlcNode);
            try {
                nodePanel.read(xmlcNode);
                jbInit();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Method declaration
     *
     */
    public void readProperties() {
        OtterXMLCNode node = null;

        if (nodeRef != null) {
            node = (OtterXMLCNode) nodeRef.get();
            nodePanel.read(node);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public HelpTopic getHelpTopic() {
        ZipHelpTopic t = new ZipHelpTopic(null, HELP_LINK);

        return t;
    }

    /**
     * Method declaration
     *
     */
    public void writeProperties() {
        nodePanel.save();
    }

    /**
     * Constructor declaration
     *
     */
    public XMLCNodePropertyPage() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //
    // PRIVATE
    //

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        this.setLayout(layoutMain);
        this.add(nodePanel, BorderLayout.CENTER);
    }

}
