/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.tool;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.common.event.HelpListener;

// Enhydra imports
import org.enhydra.xml.xmlc.XMLCException;

// JBuilder imports
import com.borland.jbuilder.JBuilderHelp;
import com.borland.jbuilder.JBuilderMenu;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.actions.ActionGroup;
import com.borland.primetime.help.ZipHelpBook;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserAction;
import com.borland.primetime.node.Project;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardPage;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import javax.swing.Action;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Container;
import java.util.ResourceBundle;

//
abstract public class AbstractBasicTool extends BrowserAction
    implements HelpListener {
    private static boolean       init = true;

    //
    public static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");    // nores

    //
    private JBProject            nativeProject = null;

    //
    public static void initAllTools(byte majorVersion, byte minorVersion) {
        if (AbstractBasicTool.init) {
            ActionGroup group;

            // standard check
            if (majorVersion != 4) {
                return;
            }

            // Initialize OpenTool here...
            System.out.println("Initializing Kelp Tools");

            // Add to Tools menu.
            group = new ActionGroup();
            group.add(new DodsTool());
            group.add(new XMLCTool());
            group.add(new DeployTool());
            JBuilderMenu.GROUP_Tools.add(group);
            init = false;
        }
    }

    //
    abstract public String getDescription();
    abstract public String getTitle();
    abstract public String getHelpAnchor();

    // implement: org.enhydra.kelp.common.event.HelpListener
    public void onHelp(HelpEvent event) {
        ZipHelpTopic topic = null;
        ZipHelpBook  book = null;
        Object       source = null;

        source = event.getSource();
        book = JBuilderHelp.createBook(Constants.KELP);
        topic = new ZipHelpTopic(book, getHelpAnchor());
        if (source instanceof JDialog) {
            topic.show((JDialog) source);
        } else if (source instanceof Component) {
            topic.show((Component) source);
        } else {
            topic.show();
        }
    }

    // Override: javax.swing.AbstractAction
    public boolean isEnabled() {
        return (Browser.getActiveBrowser().getActiveProject() != null);
    }

    // Override: javax.swing.AbstractAction
    public Object getValue(String key) {
        if (key.equals(Action.NAME)) {
            return getTitle();
        } else if (key.equals(Action.SHORT_DESCRIPTION)) {
            return getTitle();
        } else if (key.equals(Action.LONG_DESCRIPTION)) {
            return getDescription();
        }
        return (super.getValue(key));
    }

    public JBProject getNativeProject() {
        return nativeProject;
    }

    public void setNativeProject(JBProject p) {
        nativeProject = p;
    }

    public void saveNodes(OtterNode[] nodes) {
        if (nodes != null) {
            for (int i = 0; i < nodes.length; i++) {
                nodes[i].save();
            }
        }
    }

    public boolean verifyAllClassPaths(Container owner) {
        boolean valid = false;

        try {
            KelpInfo.verifyIDEClassPath();
            valid = true;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(owner, e.getMessage(),
                                          KelpInfo.getMessageBoxTitle(),
                                          JOptionPane.WARNING_MESSAGE);
        }
        if (valid) {
            valid =
                nativeProject.getPaths().putClassOnFullPath(ToolBoxInfo.XMLC_VERSION_CLASS);
            if (!valid) {
                StringBuffer buf = new StringBuffer();

                buf.append(res.getString("EnhydraLibError1"));
                buf.append('\n');
                buf.append(res.getString("EnhydraLibError2"));
                JOptionPane.showMessageDialog(owner, buf.toString(),
                                              KelpInfo.getMessageBoxTitle(),
                                              JOptionPane.WARNING_MESSAGE);
            }
        }
        return (valid);
    }

}
