/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.tool;

// JBuilder imports
import com.borland.jbuilder.JBuilderHelp;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.help.ZipHelpBook;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserAction;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.swing.KelpIconSet;
import org.enhydra.kelp.common.xmlc.CoreXMLCTool;
import org.enhydra.kelp.jbuilder.wizard.AbstractBasicWizard;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

//
public class XMLCTool extends AbstractBasicTool {

    // string not to be resourced
    private final String HELP_LINK = KelpInfo.getAddinHelpURL();    // nores

    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        AbstractBasicTool.initAllTools(majorVersion, minorVersion);
    }

    public XMLCTool() {
        putValue(BrowserAction.SMALL_ICON, KelpIconSet.getSmallIcon());
    }

    // implement: org.enhydra.kelp.jbuilder.AbstractBasicTool
    public String getTitle() {
        return CoreXMLCTool.getDefaultTitle();
    }


    // implements: org.enhydra.kelp.jbuilder.tool.AbstractBasicTool
    public String getDescription() {
        return res.getString("Compile_XMLC_DOCS");
    }

    // implement: com.borland.primetime.ide.BrowserAction
    public void actionPerformed(Browser browser) {
        PrimeProject    primeProject = null;
        CoreXMLCTool coreTool = null;

        setNativeProject((JBProject) browser.getProjectView().getActiveProject());
        if (getNativeProject() == null) {

            // do nothing
        } else {
            primeProject = new PrimeProject(getNativeProject());
            primeProject.save();
            saveNodes(primeProject.getAllXMLCNodes());
            if (verifyAllClassPaths(browser.getParent())) {
                coreTool = new CoreXMLCTool();
                coreTool.addHelpListener(this);
                coreTool.setProject(primeProject);
                coreTool.showDialog(browser);
            }
        }
    }

    public String getHelpAnchor() {
        return HELP_LINK;
    }

}
