package org.enhydra.kelp.ant.node;

//Kelp
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
//ToolBox
import org.enhydra.tool.common.PathHandle;
//Java
import java.io.File;
import java.util.ArrayList;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import java.io.IOException;

/**
 * <p>Title: </p>
 * <p>Description: This class is used by AntProject. It read all files from
 * resource directory using extension filtering and creates Document nodes
 * array. All necessary data are provided by AntProject.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Damir Milovic
 * @version 1.0
 */

public class AntNodeReader {
    private ArrayList    list = new ArrayList();
    //private OtterProject otterProject = null;
    private AntProject project = null;
    private AntNodeFactory nodeFactory = null;

    public AntNodeReader(AntProject p) {
        project = p;
        nodeFactory = (AntNodeFactory)project.getNodeFactory();
    }

    /**
     * Read all files from resource directory and create Document nodes using
     * extension filtering.
     * @param rootDir root directory.
     * @return array of all document nodes used for xml compiling.
     */
    public OtterDocumentNode[] getDocumentNodes(String rootDir){
        OtterDocumentNode[] nodes = new OtterDocumentNode[0];
        list.clear();
        //Read resources dir
        PathHandle resourcesPathHandle = PathHandle.createPathHandle(rootDir);
        OtterFolderNode rootFolder = nodeFactory.createFolderNode(project,resourcesPathHandle.getPath());
        String[] docTypes = project.getDocTypes();
        readDocument(rootFolder, docTypes); //fill the list
        nodes = new OtterDocumentNode[list.size()];
        nodes = (OtterDocumentNode[]) list.toArray(nodes);
        list.clear();
        return nodes;

    }
    /**
     * Read all files from root directory and create TextFile nodes
     * @param rootDir root directory.
     * @return array of all text files inside rootDir
     */
    public OtterTextFileNode[] getTextFileNodes(String rootDir){
        OtterTextFileNode[] nodes = new OtterTextFileNode[0];
        list.clear();
        PathHandle rootPathHandle = PathHandle.createPathHandle(rootDir);
        OtterFolderNode rootFolder = nodeFactory.createFolderNode(project,rootPathHandle.getPath());
        readTextFile(rootFolder);
        nodes = new OtterTextFileNode[list.size()];
        nodes = (OtterTextFileNode[]) list.toArray(nodes);
        list.clear();
        return nodes;
    }

    private void readDocument(OtterNode container,String[] extFilters) {

          PathHandle current = PathHandle.createPathHandle(((OtterFileNode)container).getFilePath());

          if(container instanceof OtterFolderNode){
                //Read all files inside this directory
                File directory = current.getFile();
                File[] files = directory.listFiles(); //folders or files
                for(int i=0;i<files.length;i++){
                      //Create node and read it
                      File newFile = files[i];
                      if(newFile.isDirectory()){ //Create OtterFolderNode
                            readDocument(nodeFactory.createFolderNode(container,newFile.getPath()),extFilters);
                      }else{  //Create OtterDocumentNode
                            readDocument(nodeFactory.createDocumentNode(container,newFile.getPath()),extFilters);
                      }
                }
          }else if(container instanceof OtterDocumentNode){
                if (current.getFile().exists() && current.hasExtension(extFilters)){
                      OtterDocumentNode docNode = null;
                      docNode = nodeFactory.createDocumentNode(container,current.getFile().getAbsolutePath());
                      checkSelected(docNode);
                      list.add(docNode);
                }
          }
    }
    //FIXME should use extensions filter to resolve text files !?
    private void readTextFile(OtterNode container){
        PathHandle current = PathHandle.createPathHandle(((OtterFileNode)container).getFilePath());

        if(container instanceof OtterFolderNode){
            //Read all files inside this directory
            File directory = current.getFile();
            File[] files = directory.listFiles(); //folders or files
            for(int i=0;i<files.length;i++){
                  //Create node and read it
                  File newFile = files[i];
                  if(newFile.isDirectory()){ //Create OtterFolderNode

                        readTextFile(nodeFactory.createFolderNode(container,newFile.getPath()));
                  }else{  //Create OtterDocumentNode
                         readTextFile(nodeFactory.createTextFileNode(container,newFile.getPath()));
                  }
            }
        }else if(container instanceof OtterTextFileNode){
                if (current.getFile().exists()){
                      OtterTextFileNode textFileNode = null;
                      textFileNode = nodeFactory.createTextFileNode(container,current.getFile().getAbsolutePath());
                      list.add(textFileNode);
                }
          }

    }
    private void checkSelected(OtterDocumentNode docNode){
        if(project.getProperty(AntProject.XMLC_INCLUDES).equalsIgnoreCase(AntProject.XMLC_RESOURCES_ALL)){
            //All documents need to compile
            docNode.setSelected(true);
        }else{
            //Just selected documents need to compile
            OtterDocumentNode[] selectedDoc = project.getSelectedDocuments();
            docNode.setSelected(false);
            //File docFile = new File(docNode.getFilePath());
            //File selectedFile = null;
            String docPath = docNode.getFilePath().replace('\\','/');
            for (int i = 0 ; i<selectedDoc.length;i++){
                String selectedPath = selectedDoc[i].getFilePath().replace('\\','/');
                if(docPath.equalsIgnoreCase(selectedPath)){
                    //found selected
                    docNode.setSelected(true);
                    break;
                }
            }
        }

    }
}
