/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.codegen;

// ToolBox imports
import org.enhydra.tool.codegen.ProjectGenerator;
import org.enhydra.tool.codegen.internal.WebAppGenerator;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;
import java.util.ResourceBundle;

//
public class WebAppUtil extends AppGenUtil {

    // string not to be resourced
    private final static String DEFAULT_COMMAND_NAME = "webapp";    // nores

    //
    public WebAppUtil() throws CodeGenException {
        super(new WebAppGenerator());
    }

    public WebAppUtil(OtterProject project) throws CodeGenException {
        super(project, new WebAppGenerator());
    }
    
    public WebAppUtil(OtterProject project, WebAppGenerator gen) throws CodeGenException {
        super(project, gen);
    }
    

    public static ProjectGenerator createGenerator() {
        return new WebAppGenerator();
    }

    public static String getCommandName() {
        String commandName = DEFAULT_COMMAND_NAME;

        if (KelpInfo.isClassPathComplete()) {
            commandName = WebAppGenerator.COMMAND_NAME;
        }
        return commandName;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public static String getDisplayName() {
        String displayName = res.getString("Web_Application");

        if (KelpInfo.isClassPathComplete()) {
            displayName = WebAppGenerator.displayName;
        }
        return displayName;
    }

}
