/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

package org.enhydra.kelp.common.deployer;

// Kelp imports
import org.enhydra.kelp.common.AbstractTool;

// ToolBox imports
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.SwingUtil;

// Standard imports
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

//
public class CoreDeployTool extends AbstractTool {
    static ResourceBundle addinRes =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    private DeployInnerPanel  innerPanel = null;
    private DeployButtonPanel buttonPanel = null;

    public static final String getDefaultTitle() {
      return "Kelp Deployer";
    }

    public static void main(String[] args) {
        SwingUtil.setLookAndFeelToSystem();
        CoreDeployTool tool = null;

        tool = new CoreDeployTool();
        tool.showDialog(new Frame());
        System.exit(0);
    }

    public CoreDeployTool() {
        super();
    }

    // override AbstractTool
    public void clearAll() {
        super.clearAll();
        innerPanel = null;
        buttonPanel = null;
    }

    // implement AbstractTool
    public String getTitle() {
      return CoreDeployTool.getDefaultTitle();
    }

    // implement AbstractTool
    public ActionListener createButtonListener() {
        return (new DeployButtonListener(this));
    }

    // implement AbstractTool
    public InnerPanel getInnerPanel() {
        if (innerPanel == null) {
            innerPanel = new DeployInnerPanel();
        }
        return innerPanel;
    }

    // implement AbstractTool
    public ButtonPanel getButtonPanel() {
        if (buttonPanel == null) {
            buttonPanel = new DeployButtonPanel();
        }
        return buttonPanel;
    }

    // implement AbstractTool
    protected String getProgressTitle() {
        return addinRes.getString("Deployment_Progress");
    }

    protected void back() {
      innerPanel.back();
    }

    protected void next() {
      innerPanel.next();
    }

    protected void deploy() {
       innerPanel.deploy(this);
    }

}
