/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.importer;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.PathHandle;

// Standard imports
import java.io.File;
import java.util.Vector;

//
public class MakefileData {
    private Vector docVector = new Vector();
    private Vector itemVector = new Vector();
    private String xmlcOptionFilePath = new String();
    private String xmlcParams = new String();

    //
    private String packPath = new String();
    private String docPath = new String();
    private String makeParentPath = new String();
    private String sourcePath = new String();
    private String rootPath = new String();

    protected MakefileData(String makeParent, String source) {
        makeParentPath = PathHandle.createPathString(makeParent);
        sourcePath = PathHandle.createPathString(source);
        rootPath = sourcePath;    // default
    }

    protected String getMakeParentPath() {
        return makeParentPath;
    }

    protected String getSourcePath() {
        return sourcePath;
    }

    protected void readRootPath(String p) {
        rootPath = PathHandle.createPathString(getMakeParentPath()
                                               + File.separator + p);
    }

    protected String getRootPath() {
        return rootPath;
    }

    protected void readXMLCOptionFilePath(String p) {
        xmlcOptionFilePath = PathHandle.createPathString(getMakeParentPath()
                + File.separator + p);
    }

    protected String getXMLCOptionFilePath() {
        return xmlcOptionFilePath;
    }

    protected void readPackagePath(String d) {
        packPath = PathHandle.createPathString(sourcePath + File.separator
                                               + d);
    }

    protected String getPackagePath() {
        return packPath;
    }

    protected String getJavaPackage() {
        String     javaPack = new String();
        PathHandle sourceHandle = null;
        PathHandle packHandle = null;
        int        index = -1;

        sourceHandle = PathHandle.createPathHandle(getSourcePath());
        packHandle = PathHandle.createPathHandle(getPackagePath());
        if (sourceHandle.parentOf(packHandle)) {
            index = packHandle.getPath().indexOf(sourceHandle.getPath());
            if (index > -1) {
                javaPack =
                    packHandle.getPath().substring(sourceHandle.getPath().length()
                                                   + 1);
                javaPack = javaPack.replace('/', '.');
            }
        }
        return javaPack;
    }

    protected void readDocPath(String p) {
        int index = -1;

        index = p.indexOf(')');
        if (index > 0) {
            p = getRootPath() + p.substring(index + 1);
        } else {
            p = getMakeParentPath() + File.separator + p;
        }
        docPath = PathHandle.createPathString(p);
    }

    protected String getDocPath() {
        return docPath;
    }

    protected void setXMLCParameters(String p) {
        if (p == null) {
            xmlcParams = new String();
        } else {
            xmlcParams = p.trim();
        }
    }

    protected String getXMLCParameters() {
        return xmlcParams;
    }

    protected void addSourceDocPath(String className) {
        String docResourcePath = new String();
        String docSourcePath = new String();

        if ((className != null) && (className.trim().length() > 0)) {
            docResourcePath = createSourceDocPath(getMakeParentPath(),
                                                  className);
            docSourcePath = createSourceDocPath(getDocPath(), className);
            File f = new File(docResourcePath);

            if (f.exists()) {
                docVector.addElement(docResourcePath);
            } else {
                f = new File(docSourcePath);
                if (f.exists()) {
                    docVector.addElement(docSourcePath);
                }
            }
        }
    }

    private String createSourceDocPath(String parentPath, String className) {
        PathHandle   handle = null;
        String       path = null;
        String[]     docTypes = ToolBoxInfo.getSupportedDocTypes();
        StringBuffer newPath = new StringBuffer();
        int          extLength = 3;

        handle = PathHandle.createPathHandle(parentPath + File.separator
                                             + className);
        path = handle.getPath();
        for (int i = 0; i < docTypes.length; i++) {
            if (handle.endsWith(docTypes[i])) {
                if (docTypes[i].length() > extLength) {
                    extLength = docTypes[i].length();
                }
            }
        }
        newPath.append(path.substring(0, path.length() - extLength));
        newPath.append('.');
        newPath.append(path.substring(path.length()
                                      - extLength).toLowerCase());
        return newPath.toString();
    }

    protected String[] getSourceDocPaths() {
        String[] paths = new String[docVector.size()];

        paths = (String[]) docVector.toArray(paths);
        return paths;
    }

    protected boolean containSourceDoc(String inPath) {
        String[]   docs = new String[0];
        PathHandle handle = null;
        boolean    contains = false;

        docs = getSourceDocPaths();
        handle = PathHandle.createPathHandle(inPath);
        for (int i = 0; i < docs.length; i++) {
            if (handle.equals(docs[i])) {
                contains = true;
                break;
            }
        }
        return contains;
    }

    protected void addItem(String i, String o) {
        MakeItemData data = new MakeItemData(i, o);

        itemVector.addElement(data);
    }

    protected MakeItemData[] getItems() {
        MakeItemData[] items = new MakeItemData[itemVector.size()];

        items = (MakeItemData[]) itemVector.toArray(items);
        return items;
    }

    protected boolean isSymbolic() {
        boolean sym = false;

        if (isXMLC()) {
            PathHandle packHandle = null;

            packHandle = PathHandle.createPathHandle(getPackagePath());
            if (packHandle.equals(getMakeParentPath())) {

                // Not symbolic
            } else {
                sym = true;
            }
        }
        return sym;
    }

    protected boolean isXMLC() {
        boolean      is = false;
        StringBuffer buf = new StringBuffer();

        buf.append(getXMLCOptionFilePath());
        buf.append(getXMLCParameters());
        buf.append(getDocPath());
        if (buf.toString().trim().length() > 0) {
            is = true;
        } else if (getSourceDocPaths().length >= 1) {
            is = true;
        } else if (getItems().length >= 1) {
            is = true;
        }
        return is;
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();

        buf.append("makeParentPath: " + getMakeParentPath() + '\n'); // nores
        buf.append("sourcePath: " + getSourcePath() + '\n'); // nores
        buf.append("rootPath: " + getRootPath() + '\n'); // nores
        buf.append("packagePath: " + getPackagePath() + '\n'); // nores
        buf.append("xmlcOptionFilePath: " + getXMLCOptionFilePath() + '\n'); // nores
        buf.append("xmlcParameters: " + getXMLCParameters() + '\n'); // nores
        buf.append("DocPath: " + getDocPath() + '\n'); // nores
        String[] paths = getSourceDocPaths();

        for (int i = 0; i < paths.length; i++) {
            buf.append("Class doc path: " + paths[i] + '\n'); // nores
        }
        MakeItemData[] ids = getItems();

        for (int i = 0; i < ids.length; i++) {
            buf.append(ids[i].toString());
            buf.append('\n');
        }
        return buf.toString();
    }

}
