/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.properties;

// ToolBox imports
import org.enhydra.tool.common.DialogPanel;
import org.enhydra.tool.common.SwingUtil;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.*;
import java.util.StringTokenizer;
import javax.swing.*;
import java.util.ResourceBundle;

//
public class XMLCParameterEditor extends DialogPanel {

    // strings not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");     // nores
    private final String        CARD_COMBO = "combo";                // nores
    private final String        CARD_TEXT = "text";                  // nores
    private final String        GENERATE_BOTH = "-generate both";    // nores

    //
    private KnownParam[]        knownParams = new KnownParam[0];
    private LocalOpOneListener  opOneListener = null;
    private LocalParamListener  comboListener = null;
    private LocalRadioListener  radioListener = null;
    private LocalButtonListener buttonListener = null;
    private GridBagLayout       layoutMain = null;
    private JRadioButton        radioDirect = null;
    private JRadioButton        radioList = null;
    private JPanel              panelParam = null;
    private JPanel              panelOpOne = null;
    private CardLayout          paramDeck = null;
    private CardLayout          opOneDeck = null;
    private JComboBox           comboParam = null;
    private JComboBox           comboOpOne = null;
    private JTextField          textParam = null;
    private JTextField          textOpOne = null;
    private JTextField          textOpTwo = null;
    private JPanel              buttonControl = null;
    private GridBagLayout       layoutControl = null;
    private JButton             buttonCancel = null;
    private JButton             buttonOK = null;
    private JPanel              panelRadio = null;
    private GridBagLayout       layoutRadio = null;

    public static void main(String[] args) {
        SwingUtil.setLookAndFeelToSystem();
        XMLCParameterEditor editor = null;

        editor = new XMLCParameterEditor();
        editor.showDialog();
        System.exit(0);
    }

    public XMLCParameterEditor() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public String getParameter() {
        String param = new String();

        if (radioList.isSelected()) {
            param = getListValue();
        } else {
            param = getDirectValue();
        }
        return param;
    }

    public void setParameter(String param) {
        String[] array = new String[0];

        array = paramToArray(param);
        if (isKnown(array)) {
            radioList.setSelected(true);
            showList(param);
        } else {
            radioDirect.setSelected(true);
            showDirect(param);
        }
        updateUI();
    }

    //
    //
    protected void clearDialog() {
        super.clearDialog();
    }

    //
    //
    private String getListValue() {
        StringBuffer buf = new StringBuffer();
        int          index = comboParam.getSelectedIndex();

        buf.append(comboParam.getSelectedItem().toString());
        buf.append(' ');
        buf.append(textOpOne.getText());
        buf.append(' ');
        buf.append(textOpTwo.getText());
        return buf.toString().trim();
    }

    private String getDirectValue() {
        return textParam.getText().trim();
    }

    private void showList(String param) {
        String[] array = new String[0];
        String[] selections = new String[0];
        int      index = 0;

        array = paramToArray(param);

        // default to -for-recomp if unknown
        if (!isKnown(array)) {
            array = paramToArray(param);
        }

        // setup paramCombo
        comboParam.setSelectedItem(array[0]);
        paramDeck.first(panelParam);
        index = comboParam.getSelectedIndex();

        // setup option one
        if (array.length > 1) {
            selections = knownParams[index].getSelections();
            textOpOne.setText(array[1]);
            if (selections.length >= 1) {
                comboOpOne.setModel(new DefaultComboBoxModel(selections));
                comboOpOne.getModel().setSelectedItem(array[1]);
                opOneDeck.first(panelOpOne);
            } else {
                textOpOne.setBackground(SystemColor.info);
                textOpOne.setEnabled(true);
                opOneDeck.last(panelOpOne);
            }
        } else {
            textOpOne.setBackground(SystemColor.control);
            textOpOne.setEnabled(false);
            textOpOne.setText(new String());
            opOneDeck.last(panelOpOne);
        }

        // setup option two
        if (array.length > 2) {
            textOpTwo.setBackground(SystemColor.info);
            textOpTwo.setEnabled(true);
            textOpTwo.setText(array[2]);
        } else {
            textOpTwo.setBackground(SystemColor.control);
            textOpTwo.setEnabled(false);
            textOpTwo.setText(new String());
        }
    }

    private void showDirect(String param) {
        textParam.setText(param);
        paramDeck.last(panelParam);
        textOpOne.setBackground(SystemColor.control);
        textOpOne.setText(new String());
        textOpOne.setEnabled(false);
        opOneDeck.last(panelOpOne);
        textOpTwo.setBackground(SystemColor.control);
        textOpTwo.setText(new String());
        textOpTwo.setEnabled(false);
    }

    private String[] paramToArray(String param) {
        StringTokenizer tokenizer = null;
        String[]        array = new String[0];
        int             count = -1;

        tokenizer = new StringTokenizer(param);
        count = tokenizer.countTokens();
        array = new String[count];
        int i = 0;

        while (tokenizer.hasMoreTokens()) {
            array[i] = tokenizer.nextToken();
            i++;
        }
        return array;
    }

    private boolean isKnown(String[] array) {
        boolean known = false;

        for (int i = 0; i < knownParams.length; i++) {
            if (knownParams[i].isValid(array)) {
                known = true;
                break;
            }
        }
        return known;
    }

    private void initOpts() {
        String[] options = new String[0];
        String[] selections = new String[0];
        int      index = 0;

        index = comboParam.getSelectedIndex();
        if (index == -1) {
            index = 0;
        }
        options = knownParams[index].getOptions();

        // setup option one
        if (options.length >= 1) {
            selections = knownParams[index].getSelections();
            textOpOne.setText(options[0]);
            if (selections.length >= 1) {
                comboOpOne.setModel(new DefaultComboBoxModel(selections));
                comboOpOne.getModel().setSelectedItem(options[0]);
                opOneDeck.first(panelOpOne);
            } else {
                textOpOne.setBackground(SystemColor.info);
                textOpOne.setEnabled(true);
                opOneDeck.last(panelOpOne);
            }
        } else {
            textOpOne.setBackground(SystemColor.control);
            textOpOne.setEnabled(false);
            textOpOne.setText(new String());
            opOneDeck.last(panelOpOne);
        }

        // setup option two
        if (options.length > 1) {
            textOpTwo.setBackground(SystemColor.info);
            textOpTwo.setText(options[1]);
            textOpTwo.setEnabled(true);
        } else {
            textOpTwo.setBackground(SystemColor.control);
            textOpTwo.setText(new String());
            textOpTwo.setEnabled(false);
        }
    }

    private void toggleParam() {
        if (radioList.isSelected()) {
            showList(getDirectValue());
        } else {
            showDirect(getListValue());
        }
        updateUI();
    }

    private void initKnown() {

        // strings not to be resourced
        final String[] SELECTION_PARSERS = {
            "tidy", "swing", "xerces"
        };    // nores
        final String[] SELECTION_VALIDATE = {
            "yes", "no"
        };    // nores
        final String[] SELECTION_GENERATE = {
            "both", "interface", "implementation", "class"
        };    // nores

        knownParams = new KnownParam[18];
        knownParams[0] = new KnownParam("-delete-class");    // nores
        knownParams[0].initOptions("classname");    // nores
        knownParams[1] = new KnownParam("-domfactory");    // nores
        knownParams[1].initOptions("classname");    // nores
        knownParams[2] = new KnownParam("-extends");    // nores
        knownParams[2].initOptions("classname");    // nores
        knownParams[3] = new KnownParam("-for-recomp");    // nores
        knownParams[4] = new KnownParam("-generate");      // nores
        knownParams[4].setSelections(SELECTION_GENERATE);
        knownParams[5] = new KnownParam("-html:encoding");    // nores
        knownParams[5].initOptions("encoding");    // nores
        knownParams[6] = new KnownParam("-html:addtagset");    // nores
        knownParams[6].initOptions("cyberstudio");    // nores
        knownParams[7] = new KnownParam("-html:addtag");    // nores
        knownParams[7].initOptions("tagname");    // nores
        knownParams[7].initOptions("flags");      // nores
        knownParams[8] = new KnownParam("-html:addattr");    // nores
        knownParams[8].initOptions("attrname");    // nores
        knownParams[9] = new KnownParam("-html:old-class-constants");    // nores
        knownParams[10] = new KnownParam("-html:old-name-constants");    // nores
        knownParams[11] = new KnownParam("-parser");    // nores
        knownParams[11].setSelections(SELECTION_PARSERS);
        knownParams[12] = new KnownParam("-urlmapping");    // nores
        knownParams[12].initOptions("orgURL", "newURL");    // nores
        knownParams[13] = new KnownParam("-urlregexmapping");    // nores
        knownParams[13].initOptions("regexp", "subst");    // nores
        knownParams[14] = new KnownParam("-urlsetting");    // nores
        knownParams[14].initOptions("id", "newURL");    // nores
        knownParams[15] = new KnownParam("-ssi");    // nores
        knownParams[16] = new KnownParam("-validate");    // nores
        knownParams[16].setSelections(SELECTION_VALIDATE);
        knownParams[17] = new KnownParam("-xcatalog");    // nores
        knownParams[17].initOptions("catalog");    // nores
    }

    private void pmInit() {
        ButtonGroup radioGroup = null;
        Dimension   size = null;

        // BUTTONS
        buttonListener = new LocalButtonListener();
        buttonOK.addActionListener(buttonListener);
        buttonCancel.addActionListener(buttonListener);

        // RADIO
        radioListener = new LocalRadioListener();
        radioGroup = new ButtonGroup();
        radioGroup.add(radioList);
        radioGroup.add(radioDirect);
        radioList.addItemListener(radioListener);
        radioDirect.addItemListener(radioListener);

        // WIDTH
        this.remove(panelOpOne);
        this.add(panelOpOne,
                 new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 500, 0));

        // TEXT
        textOpOne.setText(new String());
        textOpTwo.setText(new String());

        // COMBO
        initKnown();
        String[] knownStrings = new String[knownParams.length];

        for (int i = 0; i < knownStrings.length; i++) {
            knownStrings[i] = knownParams[i].toString();
        }
        comboParam.setModel(new DefaultComboBoxModel(knownStrings));

        // defaults
        String[] array = paramToArray(GENERATE_BOTH);

        comboParam.setSelectedItem(array[0]);
        textParam.setText(GENERATE_BOTH);
        setParameter(GENERATE_BOTH);

        // Start listening
        comboListener = new LocalParamListener();
        comboParam.addItemListener(comboListener);
        opOneListener = new LocalOpOneListener();
        comboOpOne.addItemListener(opOneListener);
        setTitle(res.getString("XMLC_Parameter"));
    }

    private void jbInit() throws Exception {
        layoutRadio =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioList =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioDirect =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        panelOpOne = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        opOneDeck =
            (CardLayout) Beans.instantiate(getClass().getClassLoader(),
                                           CardLayout.class.getName());
        panelParam = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        paramDeck =
            (CardLayout) Beans.instantiate(getClass().getClassLoader(),
                                           CardLayout.class.getName());
        comboParam =
            (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                          JComboBox.class.getName());
        textParam =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        textOpOne =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        comboOpOne =
            (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                          JComboBox.class.getName());
        textOpTwo =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonControl =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutControl =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonCancel =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonOK = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        panelRadio = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());

        //
        radioDirect.setText(res.getString("radioDirect_Text"));
        radioList.setSelected(true);
        radioList.setText(res.getString("radioList_Text"));
        buttonCancel.setPreferredSize(new Dimension(100, 27));
        buttonCancel.setText(res.getString("Cancel"));
        buttonOK.setPreferredSize(new Dimension(100, 27));
        buttonOK.setText(res.getString("OK"));

        //
        panelParam.setLayout(paramDeck);
        comboOpOne.setLightWeightPopupEnabled(false);
        panelParam.add(comboParam, CARD_COMBO);
        panelParam.add(textParam, CARD_TEXT);

        //
        panelOpOne.setLayout(opOneDeck);
        panelOpOne.add(comboOpOne, CARD_COMBO);
        panelOpOne.add(textOpOne, CARD_TEXT);

        //
        panelRadio.setLayout(layoutRadio);
        panelRadio.add(radioList,
                       new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.NONE,
                                              new Insets(5, 5, 5, 5), 0, 0));
        panelRadio.add(radioDirect,
                       new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.NONE,
                                              new Insets(5, 5, 5, 5), 0, 0));

        //
        buttonControl.setLayout(layoutControl);
        buttonControl.add(buttonCancel,
                          new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(5, 5, 5, 5), 0,
                                                 0));
        buttonControl.add(buttonOK,
                          new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(5, 5, 5, 5), 0,
                                                 0));
        this.setLayout(layoutMain);
        this.add(panelParam,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelOpOne,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(textOpTwo,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(buttonControl,
                 new GridBagConstraints(0, 4, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelRadio,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

    private class LocalRadioListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                toggleParam();
            }
        }

    }

    //
    private class LocalParamListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                initOpts();
            }
        }

    }

    //
    private class LocalOpOneListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                textOpOne.setText(comboOpOne.getModel().getSelectedItem().toString());
            }
        }

    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonOK) {
                setOption(DialogPanel.OK_OPTION);
                clearDialog();
            } else if (source == buttonCancel) {
                setOption(DialogPanel.CANCEL_OPTION);
                clearDialog();
            }
        }

    }

    //
    private class KnownParam {
        private String   param = new String();
        private String[] selections = new String[0];
        private String[] ops = new String[0];

        protected KnownParam(String p) {
            param = p;
        }

        public String toString() {
            return param;
        }

        protected void initOptions(String o) {
            ops = new String[1];
            ops[0] = o;
        }

        protected void initOptions(String one, String two) {
            ops = new String[2];
            ops[0] = one;
            ops[1] = two;
        }

        protected String[] getOptions() {
            return ops;
        }

        protected void setSelections(String[] s) {
            selections = s;
            ops = new String[1];
            ops[0] = selections[0];
        }

        protected String[] getSelections() {
            return selections;
        }

        protected boolean isValid(String[] array) {
            boolean valid = false;

            if (array.length == (ops.length + 1)) {
                if (param.equals(array[0])) {
                    if (selections.length == 0) {
                        valid = true;
                    } else {
                        for (int i = 0; i < selections.length; i++) {
                            if (selections[i].equals(array[1])) {
                                valid = true;
                                break;
                            }
                        }
                    }
                }
            }
            return valid;
        }

    }
}
