
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

//
import org.enhydra.tool.common.PathHandle;

//
import java.io.File;
import java.io.FileFilter;

//
public class ClassFilter implements FileFilter {
    // NFS
    protected static final String TEMP_NFS = ".nfs";                  // nores

    //
    private final String MANIFEST = "MANIFEST.MF";           // nores

    // JBuilder/JDevelper
    private final String GEN_SOURCE = "Generated Source";    // nores
    private final String DEP_CACHE = "dependency cache";     // nores

    // Forte
    private final String TYPE_JAVA = ".java";                // nores
    private final String TYPE_JSP = ".jsp";                  // nores

    //
    private PathHandle   root = null;


    //
    public ClassFilter() {}

    ;
    public ClassFilter(String r) {
        setRoot(r);
    }

    public void setRoot(String r) {
        root = PathHandle.createPathHandle(r);
    }

    public String getRoot() {
        return root.getPath();
    }

    public boolean accept(File file) {
        boolean    include = false;
        PathHandle filePath = null;

        filePath = PathHandle.createPathHandle(file);
        if (root.parentOf(filePath)) {
            if (filePath.isDirectory()) {
                include = true;
            } else {
                String relative = null;

                relative =
                    file.getAbsolutePath().substring(root.getPath().length()
                                                     + 1);
                if (relative.startsWith(GEN_SOURCE)) {

                    // exclude JB/JDev Generated Source
                } else if (relative.startsWith(DEP_CACHE)) {

                    // exclude JBuilder dependancy cache
                } else if (relative.toUpperCase().endsWith(MANIFEST)) {

                    // exclude Manifest
                } else if (relative.toLowerCase().endsWith(TYPE_JAVA)) {

                    // exclude Java source
                } else if (relative.toLowerCase().endsWith(TYPE_JSP)) {

                    // exclude JSP
                } else if (file.getName().toLowerCase().startsWith(ClassFilter.TEMP_NFS)) {

                    // exclude nfs temp files
                } else {
                    include = true;
                }
            }
        }
        return include;
    }

}
