
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// Toolbox Imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.PathHandle;

// Standard imports
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

//
public class JarPlan {

    // not to be resourced
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.archive.Res");    // nores

    //
    protected ArrayList   fileList = new ArrayList();
    private FileFilter    fillFilter = null;
    private String        archivePath = new String();
    private String        manifestPath = null;
    private String        classRoot = null;
    private String[]      classFiles = null;
    private String[]      libFiles = new String[0];
    private boolean       validate = true;
    private boolean       classFiltering = false;
    private Descriptor[]  dd = new Descriptor[0];

    public Descriptor[] getDescriptors() {
        return dd;
    }

    public void setDescriptors(Descriptor[] d) {
        dd = d;
    }

    public FileFilter getFillFilter() {
        return fillFilter;
    }

    public void setFillFilter(FileFilter f) {
        fillFilter = f;
    }

    public boolean isClassFiltering() {
        return classFiltering;
    }

    public void setClassFiltering(boolean b) {
        classFiltering = b;
    }

    public String getManifestPath() {
        return manifestPath;
    }

    public void setManifestPath(String in) throws ArchiveException {
        if (in == null) {
            manifestPath = null;
        } else {
            PathHandle ph = PathHandle.createPathHandle(in);

            if (ph.isDirectory()) {
                String exMess = "Manifest path is a directory: {0}";

                exMess = ResUtil.format(exMess, ph);
                throw new ArchiveException(exMess);
            }
            if (isValidate()) {
                if (!ph.isFile()) {
                    String exMess = "Manifest not found: {0}";

                    exMess = ResUtil.format(exMess, ph);
                    throw new ArchiveException(exMess);
                }
            }
            manifestPath = ph.getPath();
            setClassFiltering(true);
        }
    }

    public boolean isValidate() {
        return validate;
    }

    public void setValidate(boolean v) {
        validate = v;
    }

    public String getArchivePath() {
        return archivePath;
    }

    public void setArchivePath(String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException(res.getString("_archive_file_is_null"));
        } else {
            PathHandle ph = PathHandle.createPathHandle(in);

            if (ph.isDirectory()) {
                String exMess = res.getString("_archive_file_is_a");

                exMess = ResUtil.format(exMess, ph);
                throw new ArchiveException(exMess);
            }
            archivePath = ph.getPath();
        }
    }

    public String getClassRoot() {
        return classRoot;
    }

    public void setClassRoot(String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException(res.getString("Classes_directory_is"));
        } else {
            PathHandle ph = PathHandle.createPathHandle(in);

            if (isValidate()) {
                if (!ph.isDirectory()) {
                    String exMess = res.getString("Classes_directory_not");

                    exMess = ResUtil.format(exMess, ph);
                    throw new ArchiveException(exMess);
                }
            }
            classRoot = ph.getPath();
        }
    }

    public void setClassFiles(String[] classes) {
        if (isEqual(getClassFileArray(), classes)) {
            classFiles = null;
        } else {
            classFiles = classes;
        }
    }

    public String[] getClassFiles() {
        return classFiles;
    }

    public void setLibFiles(String[] libs) {
        libFiles = libs;
    }

    public String[] getLibFiles() {
        return libFiles;
    }

    public File[] getLibFileArray() {
        File[] files = new File[0];

        fileList.clear();
        for (int i = 0; i < getLibFiles().length; i++) {
            File cursor = new File(getLibFiles()[i]);

            if (cursor.isFile()) {
                fileList.add(cursor);
            }
        }
        fileList.trimToSize();
        files = new File[fileList.size()];
        files = (File[]) fileList.toArray(files);
        fileList.clear();
        return files;
    }

    public File[] getClassFileArray() {
        File[]    files = new File[0];
        NFSFilter nfsFilter = new NFSFilter();

        fileList.clear();
        if (isClassFiltering()) {
            fillFilter = new ClassFilter(getClassRoot());
        } else {
            fillFilter = null;
        }
        if (getClassRoot() == null) {

            // done
        } else if (classFiles == null) {
            fillFileList(new File(getClassRoot()));
        } else {
            for (int i = 0; i < classFiles.length; i++) {
                File cursor = new File(classFiles[i]);

                if (fillFilter == null) {
                    if (nfsFilter.accept(cursor)) {
                        fileList.add(cursor);
                    }
                } else if (fillFilter.accept(cursor)) {
                    fileList.add(cursor);
                }
            }
        }
        fileList.trimToSize();
        files = new File[fileList.size()];
        files = (File[]) fileList.toArray(files);
        fileList.clear();
        return files;
    }

    public boolean equals(Object comp) {
        boolean equal = false;
        JarPlan plan = null;

        if (comp instanceof JarPlan) {
            plan = (JarPlan) comp;
            equal = true;
            if (plan.getArchivePath() == null && getArchivePath() == null) {}
            else if (plan.getArchivePath().equals(getArchivePath())) {}
            else {
                equal = false;
            }
            if (equal) {
                if (plan.getClassRoot() == null && getClassRoot() == null) {}
                else if (plan.getClassRoot().equals(getClassRoot())) {}
                else {
                    equal = false;
                }
            }
            if (equal) {
                if (!Arrays.equals(plan.getClassFiles(), getClassFiles())) {
                    equal = false;
                }
            }
            if (equal) {
                if (!Arrays.equals(plan.getLibFiles(), getLibFiles())) {
                    equal = false;
                }
            }
            if (equal) {
                if (!Arrays.equals(plan.getDescriptors(), getDescriptors())) {
                    equal = false;
                }
            }
        }
        return equal;
    }

    // /
    protected boolean isEqual(File[] compFiles, String[] paths) {
        boolean equal = false;

        if (paths == null) {
            equal = false;
        } else if (compFiles.length == paths.length) {
            String[] comp = new String[paths.length];

            for (int i = 0; i < paths.length; i++) {
                paths[i] = PathHandle.createPathString(paths[i]);
                comp[i] = PathHandle.createPathString(compFiles[i]);
            }
            Arrays.sort(paths);
            Arrays.sort(comp);
            equal = Arrays.equals(paths, comp);
        }
        return equal;
    }

    protected void fillFileList(File root) {
        if (root.isDirectory()) {
            File[] children = new File[0];

            if (fillFilter == null) {
                children = root.listFiles(new NFSFilter());
            } else {
                children = root.listFiles(fillFilter);
            }
            for (int i = 0; i < children.length; i++) {
                if (children[i].isDirectory()) {
                    fillFileList(children[i]);
                } else if (children[i].isFile()) {
                    fileList.add(children[i]);
                }
            }
        }
    }

    private class NFSFilter implements FileFilter {

        //
        public NFSFilter() {}

        public boolean accept(File file) {
            boolean include = false;

            if (file.isDirectory()) {
                include = true;
            } else if (file.getName().toLowerCase().startsWith(ClassFilter.TEMP_NFS)) {
                include = false;
            } else {
                include = true;
            }
            return include;
        }

    }
}
