
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

//
package org.enhydra.tool.archive.wizard;

//
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.SwingUtil;

//
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.io.File;
import javax.swing.*;

//
public class OneColumnFileSelector extends RefPanel {
    private JScrollPane         scrollPane = null;
    private JList               list = null;
    private JButton             buttonAdd = null;
    private JButton             buttonRemove = null;
    private GridBagLayout       layoutMain = null;
    private LocalButtonListener buttonListener = null;
    private String[]            extensions = {
        "jar"
    };
    private String              title = "Library";

    //
    public OneColumnFileSelector() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public String[] getSelections() {
        int      size = list.getModel().getSize();
        String[] selects = new String[size];

        for (int i = 0; i < size; i++) {
            selects[i] = list.getModel().getElementAt(i).toString();
        }
        return selects;
    }

    protected void setTitle(String t) {
        title = t;
    }

    protected String getTitle() {
        return title;
    }

    protected void setExtensions(String[] exts) {
        extensions = exts;
    }

    protected String[] getExtensions() {
        return extensions;
    }

    protected void setSelections(String[] files) {
        getListModel().clear();
        for (int i = 0; i < files.length; i++) {
            getListModel().addElement(files[i]);
        }
    }

    protected JScrollPane getScrollPane() {
        return scrollPane;
    }

    //
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonAdd.addActionListener(buttonListener);
        buttonRemove.addActionListener(buttonListener);
        list.setModel(new DefaultListModel());
    }

    private void jbInit() throws Exception {
        scrollPane =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        list = (JList) Beans.instantiate(getClass().getClassLoader(),
                                         JList.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonAdd.setPreferredSize(new Dimension(75, 27));
        buttonAdd.setMargin(new Insets(2, 10, 2, 10));
        buttonAdd.setText("Add...");
        buttonRemove.setPreferredSize(new Dimension(75, 27));
        buttonRemove.setMargin(new Insets(2, 10, 2, 10));
        buttonRemove.setText("Remove");
        scrollPane.getViewport().add(list, null);
        this.setLayout(layoutMain);
        this.add(scrollPane,
                 new GridBagConstraints(0, 0, 1, 2, 0.8, 0.8,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 0, 5), 0, 0));
        this.add(buttonAdd,
                 new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.SOUTH,
                                        GridBagConstraints.NONE,
                                        new Insets(10, 5, 10, 5), 20, 0));
        this.add(buttonRemove,
                 new GridBagConstraints(1, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.NORTH,
                                        GridBagConstraints.NONE,
                                        new Insets(10, 5, 10, 5), 20, 0));
    }

    private void addChoice() {
        ExtensionFilter filter = null;
        File            choice = null;
        PathHandle      path = null;

        filter = new ExtensionFilter();
        filter.setDescriptionTitle(getTitle());
        for (int i = 0; i < getExtensions().length; i++) {
            filter.addExtension(getExtensions()[i]);
        }
        choice = SwingUtil.getFileChoice(this, getWorkingRoot(), filter,
                                         "Select a "
                                         + getTitle().toLowerCase()
                                         + " to add");
        path = PathHandle.createPathHandle(choice);
        if (path.isFile()) {
            addToModel(path);
        }
    }

    protected void addToModel(PathHandle path) {
        if (!getListModel().contains(path.getPath())) {
            getListModel().addElement(path.getPath());
        }
    }

    private DefaultListModel getListModel() {
        return (DefaultListModel) list.getModel();
    }

    protected void removeSelection() {
        int index = list.getSelectedIndex();

        if ((index > -1) && (index < getListModel().size())) {
            getListModel().remove(index);
        }
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonAdd) {
                addChoice();
            } else if (source == buttonRemove) {
                removeSelection();
            }
        }

    }
}
