
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// Standard imports
import java.io.File;

//
public class PathHandle {
    private String path = null;

    public static PathHandle createPathHandle(File inFile) {
        PathHandle handle = null;

        if (inFile == null) {
            handle = new PathHandle();
        } else {
            handle = PathHandle.createPathHandle(inFile.getAbsolutePath());
        }
        return handle;
    }

    public static PathHandle createPathHandle(String inPath) {
        PathHandle handle = null;

        handle = new PathHandle();
        handle.setPath(inPath);
        return handle;
    }

    public static String createPathString(String inPath) {
        PathHandle handle = null;

        handle = PathHandle.createPathHandle(inPath);
        return handle.getPath();
    }

    public static String createPathString(File inFile) {
        PathHandle handle = null;

        handle = PathHandle.createPathHandle(inFile);
        return handle.getPath();
    }

    public PathHandle() {}

    public boolean isFile() {
        return FileUtil.isFile(getPath());
    }

    public boolean isDirectory() {
        return FileUtil.isDirectory(getPath());
    }

    public PathHandle getParent() {
        PathHandle parent = null;
        File       parentFile = null;

        if (getFile() == null) {
            parent = null;
        } else {
            parentFile = getFile().getParentFile();
            if (parentFile == null) {
                parent = null;
            } else {
                parent = PathHandle.createPathHandle(parentFile);
            }
        }
        return parent;
    }

    public File getFile() {
        File file = null;

        if (getPath() == null) {

            // done
        } else {
            file = new File(getPath());
        }
        return file;
    }

    public void setPath(final String inPath) {
        if (inPath == null) {

            // done
        } else if (inPath.trim().length() == 0) {
            path = new String();
        } else {
            path = new String(inPath);
            if ((path.indexOf('\\') == -1) && (path.indexOf('/') == -1)) {
                path = "./" + path;
            }
            path = FileUtil.toCanonicalPath(path);
            path = path.replace('\\', '/').trim();
        }
    }

    public String getPath() {
        return path;
    }

    public String toString() {
        return getPath();
    }

    public boolean parentOf(File childFile) {
        PathHandle childHandle = null;

        childHandle = PathHandle.createPathHandle(childFile);
        return parentOf(childHandle);
    }

    public boolean parentOf(String childPath) {
        PathHandle childHandle = null;

        childHandle = PathHandle.createPathHandle(childPath);
        return parentOf(childHandle);
    }

    public boolean parentOf(PathHandle childHandle) {
        boolean isParent = false;

        if (getPath() == null || childHandle.getPath() == null) {

            // Done
        } else {
            File parentFile = null;

            parentFile = new File(getPath());
            if (parentFile.isDirectory()) {
                isParent =
                    childHandle.getComparePath().startsWith(getComparePath()
                                                            + '/');
            }
        }
        return isParent;
    }

    public boolean equals(String comparePath) {
        PathHandle compareHandle = null;

        compareHandle = PathHandle.createPathHandle(comparePath);
        return equals(compareHandle);
    }

    public boolean equals(PathHandle compare) {
        boolean equal = false;

        if (getPath() == null || compare == null
                || compare.getPath() == null) {

            // done.
        } else {
            equal = getComparePath().equals(compare.getComparePath());
        }
        return equal;
    }

    public boolean hasExtension(String[] exts) {
        boolean has = false;

        for (int i = 0; i < exts.length; i++) {
            if (hasExtension(exts[i])) {
                has = true;
                break;
            }
        }
        return has;
    }

    public boolean hasExtension(String ext) {
        boolean has = false;

        if (getPath() == null || ext == null) {

            // done
        } else {
            if (isWindows()) {
                has = ext.equalsIgnoreCase(getExtension());
            } else {
                has = ext.equals(getExtension());
            }
        }
        return has;
    }

    public boolean endsWith(String[] inPaths) {
        boolean endsWith = false;

        if (inPaths == null) {

            // done
        } else {
            for (int i = 0; i < inPaths.length; i++) {
                if (endsWith(inPaths[i])) {
                    endsWith = true;
                    break;
                }
            }
        }
        return endsWith;
    }

    public boolean endsWith(String inPath) {
        boolean endsWith = false;

        if (inPath == null || getPath() == null) {

            // done
        } else {
            inPath = inPath.replace('\\', '/');
            if (isWindows()) {
                endsWith = getComparePath().endsWith(inPath.toLowerCase());
            } else {
                endsWith = getComparePath().endsWith(inPath);
            }
        }
        return endsWith;
    }

    public boolean isEmpty() {
        boolean empty = false;

        if (getPath() == null) {
            empty = true;
        } else {
            empty = (getPath().trim().length() == 0);
        }
        return empty;
    }

    public String getExtension() {
        String ext = new String();
        int    index = -1;

        if (getPath() == null) {

            // done
        } else {
            index = getPath().lastIndexOf('.');
            if (index > getPath().lastIndexOf('/')) {
                if ((index > -1) && (getPath().length() > index)) {
                    ext = getPath().substring(index + 1);
                }
            }
        }
        return ext;
    }

    public void setExtension(String newExt) {
        String ext = new String();

        if (getPath() == null) {

            // ignore
        } else {
            ext = getExtension();
            StringBuffer buf = new StringBuffer();

            if (ext.length() > 0) {
                buf.append(path.substring(0, path.length() - ext.length()));
            } else {
                buf.append(path);
                buf.append('.');
            }
            buf.append(newExt);
            setPath(buf.toString());
        }
    }

    public PathHandle expandRelativePath(String inPath) {
        PathHandle out = null;
        PathHandle dir = null;

        if (isFile()) {
            dir = getParent();
        } else {
            dir = this;
        }
        if (inPath == null) {
            out = null;
        } else if (dir.getPath() == null) {
            out = PathHandle.createPathHandle(inPath);
        } else {
            StringBuffer outPath = new StringBuffer();

            inPath = inPath.replace('\\', '/');
            if ((inPath.length() == 1) && (inPath.charAt(0) == '.')) {
                outPath.append(dir.getPath());
            } else if ((inPath.length() > 0) && (inPath.charAt(0) == '.')) {

                // expand path relative to the project.
                outPath.append(dir.getPath());
                outPath.append('/');
                if ((inPath.length() > 1) && (inPath.charAt(1) == '.')) {
                    outPath.append(inPath);
                } else {
                    outPath.append(inPath.substring(2));
                }
            } else {
                outPath.append(inPath);
            }
            out = PathHandle.createPathHandle(outPath.toString());
        }
        return out;
    }

    public String getRelativePath(String in) {
        String relPath = new String();
        if ((in.length() > 0) && (in.charAt(0) == '.')) {
           relPath = in.replace('\\','/');
        } else {
           relPath = getRelativePath(PathHandle.createPathHandle(in));
        }
        return relPath;
    }

    public String getRelativePath(PathHandle inPath) {
        String     relPath = new String();
        PathHandle dir = null;

        if (isFile()) {
            dir = getParent();
        } else {
            dir = this;
        }
        if (dir.getPath() == null) {
            relPath = inPath.getPath();
        } else {
            StringBuffer relBuf = new StringBuffer();

            if (dir.equals(inPath)) {
                relBuf.append('.');
                relBuf.append('/');
            } else if (dir.parentOf(inPath)) {
                relBuf.append('.');
                relBuf.append('/');
                relBuf.append(inPath.getPath().substring(dir.getPath().length()
                                                         + 1));
            } else {
                relBuf.append(inPath);
            }
            relPath = relBuf.toString();
        }

        // Return with separators set to / for any OS but don't do
        // path handle to avoid decompress.
        return relPath.replace('\\', '/');
    }

    private String getComparePath() {
        String thisPath = getPath();

        if (getPath() == null) {

            // done
        } else if (isWindows()) {
            thisPath = getPath().toLowerCase();
        } else {
            thisPath = new String(getPath());
        }
        return thisPath;
    }

    private boolean isWindows() {
        return File.separatorChar == '\\';
    }

}
