
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// standard imports
import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Enumeration;

//
public class Template {
    private File   output = null;
    private URL    url = null;
    private String base = new String();

    //
    public Template(URL u, String b) {
        url = u;
        base = b.replace('\\', '/');
    }

    public Template(File f, String b) {
        try {
            url = f.toURL();
            base = b.replace('\\', '/');
        } catch (MalformedURLException e) {
            e.printStackTrace(System.err);
        }
    }

    public InputStream getInputStream() throws IOException {
        return url.openStream();
    }

    public Template[] listTemplates() {
        Template[] children = new Template[0];

        children = listTemplates(null);
        return children;
    }

    public Template[] listTemplates(TemplateFilter filter) {
        Template[] children = new Template[0];
        ArrayList  list = null;
        Template[] jarTemplates = new Template[0];
        Template[] fileTemplates = new Template[0];

        jarTemplates = listJarTemplates(filter);
        fileTemplates = listFileTemplates(filter);
        list = new ArrayList(Arrays.asList(fileTemplates));
        for (int j = 0; j < jarTemplates.length; j++) {
            boolean override = false;

            for (int f = 0; f < fileTemplates.length; f++) {
                if (jarTemplates[j].getRelativePath().equals(fileTemplates[f].getRelativePath())) {
                    override = true;
                    break;
                }
            }
            if (!override) {
                list.add(jarTemplates[j]);
            }
        }
        jarTemplates = new Template[0];
        fileTemplates = new Template[0];
        list.trimToSize();
        if (list.size() > 0) {
            children = new Template[list.size()];
            children = (Template[]) list.toArray(children);
        }
        list.clear();
        return children;
    }

    private Template[] listJarTemplates(TemplateFilter filter) {
        Template[]       children = new Template[0];
        ArrayList        list = new ArrayList();
        JarURLConnection connect = null;
        JarFile          jar = null;
        JarEntry         entry = null;
        Enumeration      entries = null;
        URL              baseUrl = null;
        int              index = -1;

        if (url.getProtocol().equals("jar")) {
            baseUrl = url;
        } else {
            StringBuffer buf = new StringBuffer();

            buf.append(getClass().getName().replace('.', '/'));
            buf.append(".class");
            baseUrl = ClassLoader.getSystemResource(buf.toString());
            buf.setLength(0);
            index = baseUrl.toString().indexOf('!');
            if (index > -1) {
                buf.append(baseUrl.toString().substring(0, index + 1));
                buf.append('/');
                buf.append(getBasePath());
                if (!buf.toString().endsWith("/")) {
                    buf.append('/');
                }
                try {
                    baseUrl = new URL(buf.toString());
                } catch (MalformedURLException e) {
                    baseUrl = null;
                    e.printStackTrace(System.err);
                }
            } else {
                baseUrl = null;
            }
        }
        if (baseUrl == null) {
            entries = null;
        } else {
            try {
                connect = (JarURLConnection) baseUrl.openConnection();
                jar = connect.getJarFile();
                entries = jar.entries();
            } catch (FileNotFoundException e) {
                System.err.println(e.getMessage());
                entries = null;
            } catch (IOException e) {
                e.printStackTrace(System.err);
                entries = null;
            }
        }
        if (entries == null) {

            //
        } else {
            while (entries.hasMoreElements()) {
                String entryName = new String();
                String parentPath = new String();

                entry = (JarEntry) entries.nextElement();
                entryName = entry.getName();

                // Exclude our own directory
                if (entryName.equals(getBasePath())) {
                    continue;
                }
                index = entryName.lastIndexOf('/');
                if (index > -1 && entry.isDirectory()) {
                    parentPath = entryName.substring(0, index);
                } else {
                    parentPath = entryName;
                }
                index = parentPath.lastIndexOf('/');
                if (index > -1) {
                    parentPath = parentPath.substring(0, index + 1);
                }
                if (!parentPath.equals(getBasePath())) {
                    continue;
                }
                Template     temp = null;
                URL          newUrl = null;
                StringBuffer buf = new StringBuffer();

                buf.append(extractJarPath(baseUrl));
                buf.append(entryName);
                try {
                    newUrl = new URL(buf.toString());
                    temp = new Template(newUrl, base);
                    if (filter == null) {
                        list.add(temp);
                    } else if (filter.accept(temp)) {
                        list.add(temp);
                    }
                } catch (MalformedURLException e) {
                    e.printStackTrace(System.err);
                }
            }
        }
        list.trimToSize();
        if (list.size() > 0) {
            children = new Template[list.size()];
            children = (Template[]) list.toArray(children);
        }
        list.clear();
        return children;
    }

    private Template[] listFileTemplates(TemplateFilter filter) {
        Template[] children = new Template[0];
        ArrayList  list = new ArrayList();
        URL        baseUrl = null;
        File       dir = null;
        File[]     files = new File[0];

        if (url.getProtocol().equalsIgnoreCase("file")) {
            baseUrl = url;
        } else if (url.toString().startsWith("jar:file:")
                   && url.toString().indexOf('!') > -1) {
            StringBuffer buf = new StringBuffer();
            String       path = url.toString();

            path = path.substring(4, url.toString().indexOf('!'));
            path = path.substring(0, path.lastIndexOf('/'));
            path = path.substring(0, path.lastIndexOf('/') + 1);
            buf.append(path);
            buf.append(getBasePath());
            try {
                baseUrl = new URL(buf.toString());
            } catch (MalformedURLException e) {
                e.printStackTrace(System.err);
                baseUrl = null;
            }
        }
        if (baseUrl == null) {
            list.clear();
        } else {
            dir = new File(baseUrl.getFile());
            if (dir.isDirectory()) {
                files = dir.listFiles();
                for (int i = 0; i < files.length; i++) {
                    Template temp = new Template(files[i], base);

                    if (filter == null) {
                        list.add(temp);
                    } else if (filter.accept(temp)) {
                        list.add(temp);
                    }
                }
            }
        }
        list.trimToSize();
        if (list.size() > 0) {
            children = new Template[list.size()];
            children = (Template[]) list.toArray(children);
        }
        list.clear();
        return children;
    }

    public int endsWith(String[] list) {
        int ew = -1;

        for (int i = 0; i < list.length; i++) {
            if (endsWith(list[i])) {
                ew = i;
                break;
            }
        }
        return ew;
    }

    public boolean endsWith(String in) {
        boolean ew = false;

        if (in == null || url == null) {
            ew = false;
        } else {
            String compEnd = new String(in);
            String compUrl = new String(url.toString());

            compEnd = compEnd.replace('\\', '/');
            if (compUrl.endsWith("/") && (!compEnd.endsWith("/"))) {
                compEnd = compEnd + '/';
            }
            compEnd = compEnd.toLowerCase();
            compUrl = compUrl.toLowerCase();
            ew = compUrl.endsWith(compEnd);
        }
        return ew;
    }

    public String getExtension() {
        String ext = new String();
        int    index = -1;

        if (url == null) {
            ext = new String();
        } else {
            ext = new String(url.toString());
            ext = ext.replace('\\', '/');
            index = ext.lastIndexOf('/');
            if (index > -1) {
                ext = ext.substring(index);
            }
            index = ext.lastIndexOf('.');
            if (index > -1 && (ext.length() > index)) {
                ext = ext.substring(index + 1);
            } else {
                ext = new String();
            }
        }
        return ext;
    }

    public String getName() {
        String n = new String();
        int    index = -1;

        if (url == null) {
            n = new String();
        } else {
            n = url.getFile();
            index = n.lastIndexOf('/');
            if (index == n.length()) {
                n = n.substring(0, n.length() - 1);
            }
            index = n.lastIndexOf('/');
            n = n.substring(index);
        }
        return n;
    }

    private String getBasePath() {
        StringBuffer buf = new StringBuffer();

        buf.append(base);
        buf.append(getRelativePath());
        return buf.toString();
    }

    private String extractJarPath(URL jarUrl) {
        StringBuffer buf = new StringBuffer();
        int          index = -1;

        if (jarUrl.getProtocol().equalsIgnoreCase("jar")) {
            index = jarUrl.toString().indexOf('!');
            if (index > -1) {
                buf.append(jarUrl.toString().substring(0, index + 1));
                buf.append('/');
            }
        }
        return buf.toString();
    }

    public String getRelativePath() {
        String path = new String();
        String file = new String();
        int    index = -1;

        if (base == null || url == null) {
            path = new String();
        } else {
            file = url.getFile();
            index = file.indexOf(base);
            if (index > -1) {
                path = file.substring(index + base.length());
            } else {
                index = file.lastIndexOf('/');
                if (index > -1) {
                    path = file.substring(index);
                } else {
                    path = file;
                }
            }
        }
        if (path.startsWith("/")) {
            if (path.length() == 1) {
                path = ".";
            } else {
                path = path.substring(1, path.length());
            }
        }
        return path;
    }

    public boolean isDirectory() {
        boolean isd = false;

        if (url == null) {
            isd = false;
        } else if (url.getProtocol().equals("file")) {
            File file = new File(url.getFile());

            isd = file.isDirectory();
        } else if (url.getProtocol().equals("jar")) {
            JarURLConnection connect;
            JarEntry         entry = null;

            try {
                connect = (JarURLConnection) url.openConnection();
                entry = connect.getJarEntry();
                isd = (entry.getSize() == 0) || entry.isDirectory()
                      || entry.getName().endsWith("/");
            } catch (IOException e) {
                e.printStackTrace(System.err);
                isd = false;
            }
        }
        return isd;
    }

    public boolean isFile() {
        boolean isf = false;

        if (url == null) {
            isf = false;
        } else if (url.getProtocol().equals("file")) {
            File file = new File(url.getFile());

            isf = file.isFile();
        } else if (url.getProtocol().equals("jar")) {
            JarURLConnection connect;

            try {
                connect = (JarURLConnection) url.openConnection();
                isf = (connect.getJarEntry().getSize() > 0);
            } catch (IOException e) {
                e.printStackTrace(System.err);
                isf = false;
            }
        }
        return isf;
    }

    public File getOutput() {
        return output;
    }

    public void setOutput(File f) {
        output = f;
    }

    public String toString() {
        return url.toString();
    }

}
