package org.enhydra.kelp.jbuilder.node;

import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.vfs.VFS;
import com.borland.primetime.vfs.Filesystem;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.FolderNode;
import com.borland.primetime.node.LightweightNode;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class NodeUtil {
  /**
   * Add needed FileNodes
   *
   * This routine finds all the files in the provided directory and adds
   * them to the given project if they are not there already.
   *
   * @param jbProject Project where nodes are to be added.
   * @param url Current directory to examine for subdirectories.
   */
  public static void doAddFiles(JBProject jbProject,
                          Node parent,
                          Url url) throws Exception {
    Url[] urls = VFS.getChildren(url, Filesystem.TYPE_FILE);
    if (urls.length > 0) {
      for (int j = 0; j < urls.length; j++) {
        if ((jbProject.findNode(urls[j]) == null)
          && checkIfVisibleFile(urls[j])) {
          FileNode node = new FileNode(jbProject, parent, urls[j]);
        }
      }
    }
  }

  /**
   * Recursively add needed files
   *
   * This routine uses recursive logic to locate all subdirectories under
   * the given root directory and (if not already present) add a FileNodes
   * for each of those subdirectories
   *
   * @param jbProject Project where nodes are to be added.
   * @param rootDir Root directory of the recursive search.
   * @param url Current directory to examine for subdirectories.
   */
  public static void doAddFolders(JBProject jbProject,
                            Node parent,
                            Url url) throws Exception {
    FolderNode node = null;
    LightweightNode nodes[] = jbProject.findNodes(url.getName());
    for (int i = 0; i < nodes.length; i++) {
      if (nodes[i].getParent() == parent) {
        node = (FolderNode)nodes[i];
      }
    }
    if (node == null)
      node = new FolderNode(jbProject, parent, url.getName());

    doAddFiles(jbProject, node, url);

    Url[] urls = VFS.getChildren(url, Filesystem.TYPE_DIRECTORY);
    if (urls.length > 0) {
      for (int j = 0; j < urls.length; j++) {
        doAddFolders(jbProject, node, urls[j]);
      }
    }
  }

    /**
     * Check if visible file.
     *
     * This routine looks at the given file to determine if it should be made
     * visible as a FileNode.  Assume everything except backup files.
     *
     * @param url Directory to examine for files.
     * @return Returns true if file should be made visible.
     */
    private static boolean checkIfVisibleFile(Url url) {
      return (url.getFileObject().getAbsolutePath().endsWith("~") != true);
    }

}