
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.EarPlan;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.archive.WebApplication;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.util.ArrayList;
import java.util.jar.Manifest;

//
public class EarType extends ArchiveType {
    public final static String EXT = "ear";

    //
    protected final String getSelectionName() {
        return "Enterprise Application";
    }

    protected final String getExtension() {
        return EXT;
    }

    protected final String getType() {
        return getExtension();
    }

    protected final String getDescription() {
        StringBuffer buf = new StringBuffer();

        buf.append("Create an enterprise application archive ");
        buf.append("consisting of modules and supporting libraries.");
        buf.append("The modules can include web archives, enterprise ");
        buf.append("JavaBean archives and other enterprise applicaion ");
        buf.append("archives.");
        return buf.toString();
    }

    protected void initPanels() throws ArchiveException {
        ArchivePanel[]        newPanels = new ArchivePanel[4];
        EarFilePanel          filePanel = new EarFilePanel();
        WebAppPanel           wPanel = new WebAppPanel();
        EnterpriseModulePanel ePanel = new EnterpriseModulePanel();
        MetaPanel             metaPanel = new MetaPanel();

        newPanels[0] = filePanel;
        newPanels[1] = wPanel;
        newPanels[2] = ePanel;
        newPanels[3] = metaPanel;
        setWizardPanels(newPanels);
    }

    protected void initPlan() throws ArchiveException {
        EarPlan    earPlan = null;
        PathHandle path = null;

        setPlan(new EarPlan());
        getEarPlan().setArchivePath(getDefaultArchivePath());
        path = PathHandle.createPathHandle(getEarPlan().getArchivePath());
        if (path.isFile()) {
            readArchive(getEarPlan().getArchivePath());
        }
    }

    protected boolean readArchive(Manifest manifest, String[] entries) {
        boolean      read = super.readArchive(manifest, entries);
        Boolean      b = null;
        PathHandle   root = null;
        PathHandle   cursor = null;
        ArrayList    list = new ArrayList();
        String[]     files = new String[0];
        Descriptor[] dd = new Descriptor[0];
        WebApplication[] webApps = new WebApplication[0];

        if (read) {
            String value = null;

            try {

                // appname
                value = manifest.getMainAttributes().getValue(APP_NAME);
                if (value != null) {
                    getEarPlan().setAppName(value);
                }

                // description
                value = manifest.getMainAttributes().getValue(APP_DESC);
                if (value != null) {
                    getEarPlan().setDescription(value);
                }

                // alt-dd
                value = manifest.getMainAttributes().getValue(ALT_DD);
                b = Boolean.valueOf(value);
                getEarPlan().setAlt(b.booleanValue());

                // copy client
                value = manifest.getMainAttributes().getValue(COPY_CLIENT);
                b = Boolean.valueOf(value);
                getEarPlan().setCopyClient(b.booleanValue());

                // enterprise modules (0 - 99)
                for (int i = 0; i < 100; i++) {
                    value = manifest.getMainAttributes().getValue(ENTERPRISE_MODULE
                            + i);
                    cursor = PathHandle.createPathHandle(value);
                    if (cursor.isFile()) {
                        list.add(cursor.getPath());
                    }
                }
                files = new String[list.size()];
                files = (String[]) list.toArray(files);
                list.clear();
                getEarPlan().setEnterpriseModules(files);

                // web apps (0 - 99)
                for (int i = 0; i < 100; i++) {
                    value = manifest.getMainAttributes().getValue(WEBAPP_MODULE
                            + i);
                    cursor = PathHandle.createPathHandle(value);
                    if (cursor.isFile()) {
                        WebApplication webApp = null;
                        webApp = new WebApplication(cursor.getPath());
                        value = manifest.getMainAttributes().getValue(WEBAPP_CONTEXT
                           + i);
                        if (value != null && value.trim().length() > 0) {
                          webApp.setContextRoot(value);
                        }
                        list.add(webApp);
                    }
                }
                webApps = new WebApplication[list.size()];
                webApps = (WebApplication[]) list.toArray(webApps);
                list.clear();
                getEarPlan().setWebApplications(webApps);


                // meta descriptors 0-99
                for (int i = 0; i < 100; i++) {
                    value = manifest.getMainAttributes().getValue(APP_META
                            + i);
                    cursor = PathHandle.createPathHandle(value);
                    if (cursor.isFile()) {
                        Descriptor desc = null;

                        desc = new Descriptor(cursor.getPath());
                        list.add(desc);
                    }
                }
                list.trimToSize();
                dd = new Descriptor[list.size()];
                dd = (Descriptor[]) list.toArray(dd);
                list.clear();
                getPlan().setDescriptors(dd);
            } catch (ArchiveException e) {
                e.printStackTrace(System.err);
            }
        }
        return read;
    }

    private EarPlan getEarPlan() throws ArchiveException {
        return (EarPlan) getPlan();
    }

}
