
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

//
// ToolBox imports
//
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.archive.EjbPlan;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

//
// Standard imports
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.jar.Manifest;
import java.util.jar.JarInputStream;

//
public class EjbType extends ArchiveType {
    public final static String EXT = "jar";

    //
    protected final String getSelectionName() {
        return "Enterprise JavaBean";
    }

    protected final String getExtension() {
        return EXT;
    }

    protected final String getType() {
        return "ejb-jar";
    }

    protected final String getDescription() {
        StringBuffer buf = new StringBuffer();

        buf.append("Create an enterprise JavaBean archive for ");
        buf.append("inclusion in an enterprise application ");
        buf.append("archive or for deployment as a shared resource.");
        return buf.toString();
    }

    protected void initPanels() throws ArchiveException {
        EjbFilePanel   filePanel = new EjbFilePanel();
        ClassesPanel   classesPanel = new ClassesPanel();
        ArchivePanel[] newPanels = new ArchivePanel[2];

        newPanels[0] = filePanel;
        newPanels[1] = classesPanel;
        setWizardPanels(newPanels);
    }

    protected void initPlan() throws ArchiveException {
        PathHandle path = null;

        setPlan(new EjbPlan());
        getEjbPlan().setArchivePath(getDefaultArchivePath());
        path = PathHandle.createPathHandle(getEjbPlan().getArchivePath());
        if (path.isFile()) {
            readArchive(getEjbPlan().getArchivePath());
        }
    }

    protected boolean readArchive(Manifest manifest, String[] entries) {
        boolean    read = super.readArchive(manifest, entries);
        Boolean    b = null;
        PathHandle root = null;

        if (read) {
            String value = null;

            try {
                // create client
                value = manifest.getMainAttributes().getValue(CREATE_CLIENT);
                b = Boolean.valueOf(value);
                getEjbPlan().setCreateClient(b.booleanValue());
            } catch (ArchiveException e) {
                e.printStackTrace(System.err);
            }
        }
        return read;
    }

    private EjbPlan getEjbPlan() throws ArchiveException {
        return (EjbPlan) getPlan();
    }

}
