
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// Standard imports
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import javax.swing.AbstractButton;
import javax.swing.JPanel;
import java.awt.*;
import java.beans.*;

//
public class ButtonPanel extends JPanel {
    public static final int      BACK = 0;
    public static final int      NEXT = 1;
    public static final int      DEPLOY = 2;
    public static final int      HELP = 3;
    public static final int      ABOUT = 4;
    public static final int      CLOSE = 5;
    public static final int      COMPILE = 6;
    public static final int      CANCEL = 7;
    public static final int      FINISH = 7;
    public static final int      GENERATE = 8;

    //
    public static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //
    public static final String   COMMAND_BACK = res.getString("_Back");
    public static final String   COMMAND_NEXT = res.getString("Next_");
    public static final String   COMMAND_DEPLOY = res.getString("Deploy");
    public static final String   COMMAND_HELP = res.getString("Help");
    public static final String   COMMAND_ABOUT = res.getString("About_");
    public static final String   COMMAND_CLOSE = res.getString("Close");
    public static final String   COMMAND_COMPILE = res.getString("Compile");
    public static final String   COMMAND_CANCEL = res.getString("Cancel");
    public static final String   COMMAND_FINISH = res.getString("Finish");
    public static final String   COMMAND_GENERATE = res.getString("Generate");

    //
    private ActionListener[]     actionListeners = new ActionListener[0];
    private GridBagLayout        layoutDefault;

    public static Component findButton(Container container, String command) {
        Component[] children = null;
        Component   button = null;

        if (container == null) {
            children = new Component[0];
        } else {
            children = container.getComponents();
        }
        for (int i = 0; i < children.length; i++) {
            if (children[i] instanceof AbstractButton) {
                AbstractButton suspect = null;

                suspect = (AbstractButton) children[i];
                if (suspect.getActionCommand().equals(command)) {
                    button = suspect;
                }
            } else if (children[i] instanceof Button) {
                Button suspect = null;

                suspect = (Button) children[i];
                if (suspect.getActionCommand().equals(command)) {
                    button = suspect;
                }
            } else if (children[i] instanceof Container) {
                button = findButton((Container) children[i], command);
            }
            if (button != null) {
                break;
            }
        }
        return button;
    }

    public ButtonPanel() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Window getWindow() {
        return (Window) getTopLevelAncestor();
    }

    public void removeHelp() {}

    public void clearAll() {
        removeAll();
        actionListeners = new ActionListener[0];
    }

    public synchronized void addActionListener(ActionListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(actionListeners));
        if (!list.contains(l)) {
            list.add(l);
        }
        list.trimToSize();
        actionListeners = new ActionListener[list.size()];
        actionListeners = (ActionListener[]) list.toArray(actionListeners);
        list.clear();
    }

    public synchronized void removeActionListener(ActionListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(actionListeners));
        if (list.contains(l)) {
            list.remove(l);
        }
        list.trimToSize();
        actionListeners = new ActionListener[list.size()];
        actionListeners = (ActionListener[]) list.toArray(actionListeners);
        list.clear();
    }

    public void notifyActionListeners(ActionEvent event) {
        for (int i = 0; i < actionListeners.length; i++) {
            actionListeners[i].actionPerformed(event);
        }
    }

    private void jbInit() throws Exception {
        layoutDefault =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        this.setLayout(layoutDefault);
    }

}
