
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// ToolBox imports
import org.enhydra.tool.common.event.CancelEvent;
import org.enhydra.tool.common.event.CancelListener;
import org.enhydra.tool.common.event.ProgressEvent;
import org.enhydra.tool.common.event.ProgressListener;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.util.Vector;
import javax.swing.*;
import java.util.ResourceBundle;

//
public class ProgressMeter extends DialogPanel implements ProgressListener {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //
    private LocalButtonListener buttonListener = null;
    private Vector              cancelVector = new Vector();
    private JPanel              panelMain = null;
    private GridBagLayout       layoutMain = null;
    private JProgressBar        progressBar = null;
    private JTextArea           textMessage = null;
    private JButton             buttonCancel = null;
    private boolean             disposeOnClose = true;
    private boolean             cancel = false;

    public static void main(String[] args) {
        SwingUtil.setLookAndFeelToSystem();
        ProgressMeter meter;

        meter = new ProgressMeter();
        meter.setStandalone(true);
        meter.startDialogThread();
        for (int i = 0; i < 101; i++) {
            ProgressEvent event = new ProgressEvent(meter, i,
                                                    new String() + i);

            try {
                Thread.sleep(100);
            } catch (Exception e) {
                e.printStackTrace();
            }
            meter.onProgress(event);
        }
    }

    public ProgressMeter() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public CancelListener[] getCancelListeners() {
        CancelListener[] cancelListeners = new CancelListener[0];

        cancelListeners = new CancelListener[cancelVector.size()];
        cancelListeners =
            (CancelListener[]) cancelVector.toArray(cancelListeners);
        return cancelListeners;
    }

    public void addCancelListener(CancelListener l) {
        if (!cancelVector.contains(l)) {
            cancelVector.addElement(l);
        }
    }

    public void removeCancelListener(CancelListener l) {
        if (cancelVector.contains(l)) {
            cancelVector.removeElement(l);
        }
    }

    private void notifyCancel() {
        CancelListener[] cancelListeners = getCancelListeners();

        if (cancelListeners.length == 0) {

            // done
        } else {
            CancelEvent event = null;

            event = new CancelEvent(this);
            for (int i = 0; i < cancelListeners.length; i++) {
                cancelListeners[i].onCancel(event);
            }
        }
    }

    public void onProgress(ProgressEvent event) {
        if (event.getProgress() > 99) {
            clearDialog();
        } else if (isCancelled()) {

            // do nothing
        } else {
            updateProgress(event.getMessage(), event.getProgress());
        }
    }

    public void setDisposeOnClose(boolean b) {
        disposeOnClose = b;
    }

    public boolean isDisposeOnClose() {
        return disposeOnClose;
    }

    public boolean isCancelled() {
        return cancel;
    }

    //
    //
    protected void clearDialog() {
        cancelVector.clear();
        cancelVector.trimToSize();
        if (getDialog() != null) {
            if (isDisposeOnClose()) {
                super.clearDialog();
            } else {
                hideDialog();
                clearProgress();
            }
        }
    }

    //
    //
    private void updateProgress(final String message, final int progress) {
        ProgressUpdate runUpdate = null;

        runUpdate = new ProgressUpdate(textMessage, message, progressBar,
                                       progress);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runUpdate.run();
            } else {
                SwingUtilities.invokeAndWait(runUpdate);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void clearProgress() {
        textMessage.setText(new String());
        progressBar.setValue(1);
        cancel = false;
    }

    private void doCancel() {
        setOption(DialogPanel.CANCEL_OPTION);
        textMessage.setText(res.getString("Cancelling_operation_"));
        cancel = true;
        notifyCancel();
    }

    private void pmInit() {
        JLabel    label = new JLabel();
        Dimension d = new Dimension();

        d.height = 25;
        d.width = 300;
        textMessage.setFont(label.getFont());
        textMessage.setPreferredSize(d);
        progressBar.getModel().setMinimum(0);
        progressBar.getModel().setMaximum(100);
        clearProgress();
        buttonListener = new LocalButtonListener();
        buttonCancel.addActionListener(buttonListener);
    }

    private void jbInit() throws Exception {
        panelMain = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        progressBar =
            (JProgressBar) Beans.instantiate(getClass().getClassLoader(),
                                             JProgressBar.class.getName());
        textMessage =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        buttonCancel =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelMain.setLayout(layoutMain);
        textMessage.setWrapStyleWord(true);
        textMessage.setLineWrap(true);
        textMessage.setRequestFocusEnabled(false);
        textMessage.setEnabled(false);
        textMessage.setDisabledTextColor(SystemColor.controlText);
        textMessage.setBackground(SystemColor.control);
        textMessage.setEditable(false);
        buttonCancel.setText(res.getString("Cancel"));
        panelMain.add(progressBar,
                      new GridBagConstraints(0, 2, 2, 1, 0.1, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 10, 20, 10), 0,
                                             0));
        panelMain.add(textMessage,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(20, 10, 5, 5), 0,
                                             10));
        panelMain.add(buttonCancel,
                      new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 10), 0, 0));
        this.add(panelMain);
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonCancel) {
                doCancel();
            }
        }

    }
    private class ProgressUpdate implements Runnable {
        int          prog = 0;
        String       mess = new String();
        JTextArea    ta = null;
        JProgressBar bar = null;

        protected ProgressUpdate(JTextArea t, String m, JProgressBar b,
                                 int p) {
            ta = t;
            mess = m;
            bar = b;
            prog = p;
        }

        //
        synchronized public void run() {
            bar.getModel().setValueIsAdjusting(true);
            if (!mess.equals(ta.getText())) {
                ta.setText(mess);
            }
            bar.getModel().setValue(prog);
            bar.getModel().setValueIsAdjusting(false);
            bar.updateUI();
            bar.repaint();
            repaint();
        }

    }
}
