
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common.wizard;

// ToolBox imports
import org.enhydra.tool.common.InnerPanel;

// Standard imports
import java.awt.*;
import javax.swing.*;
import java.beans.Beans;
//
public class TBWizardInnerPanel extends InnerPanel {
    private final String  FILE_WIZARD_JPG =
        "org/enhydra/tool/common/media/lutris/otterwizard.jpg";    // nores
    private BorderLayout  layoutImage;
    private boolean       showImage = true;
    private GridBagLayout layoutMain = null;
    private JPanel        panelImage;
    private JLabel        labelImage;
    private TBWizardDeck  deck = null;

    public TBWizardInnerPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public TBWizardDeck getDeck() {
        return deck;
    }

    public void setDeck(TBWizardDeck d) {
        Component[] comps = new Component[0];

        deck = d;
        deck.setMinimumSize(new Dimension(100, 100));
        deck.setPreferredSize(new Dimension(100, 100));
        comps = getComponents();
        for (int i = 0; i < comps.length; i++) {
            if (comps[i] instanceof TBWizardDeck) {
                remove(comps[i]);
            }
        }
        this.add(deck,
            new GridBagConstraints(1, 0, 1, 1, 0.9, 0.9,
                                   GridBagConstraints.CENTER,
                                   GridBagConstraints.BOTH,
                                   new Insets(2, 2, 2, 2), 0, 0));

    }

    public void setLargeIcon(Icon icon) {
        if (icon == null) {
            panelImage.setVisible(false);
            labelImage.setIcon(null);
        } else {
            panelImage.setVisible(true);
            labelImage.setText(new String());
            labelImage.setIcon(icon);
        }
    }

    public Icon getLargeIcon() {
        return labelImage.getIcon();
    }

    // override InnerPanel
    public void save() {
       // do nothing on purpose
    }


    // override InnerPanel
    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[0];
        TBWizardPage pageOne = null;
        if (getDeck() == null) {
          comps = super.getFirstFocusComponents();
        } else if (getDeck().getPages().length > 0) {
          pageOne = getDeck().getPages()[0];
          comps = pageOne.getChildPanelComponents();
        } else {
          comps = super.getFirstFocusComponents();
        }
        return comps;
    }


    /**
     * Add a page to the wizard.
     *
     * @param page
     * A panel that contains swing controls that
     * are specific to the current wizard.
     */
    public void addWizardPage(TBWizardPage page) {
        deck.addWizardPage(page);
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        deck = (TBWizardDeck) Beans.instantiate(getClass().getClassLoader(),
                                                TBWizardDeck.class.getName());
        panelImage = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        layoutImage =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        labelImage = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                JLabel.class.getName());
        panelImage.setLayout(layoutImage);
        panelImage.add(labelImage, BorderLayout.CENTER);
        this.setLayout(layoutMain);
        this.add(panelImage,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 2, 2, 2), 0, 0));
        this.add(deck,
            new GridBagConstraints(1, 0, 1, 1, 0.9, 0.9,
                                   GridBagConstraints.CENTER,
                                   GridBagConstraints.BOTH,
                                   new Insets(2, 2, 2, 2), 0, 0));

    }

    private void pmInit() {
        setDeck(deck);
        ImageIcon image = null;

        try {
            image =
                new ImageIcon(ClassLoader.getSystemResource(FILE_WIZARD_JPG));
        } catch (NullPointerException e) {
            image = null;
        }
        setLargeIcon(image);
    }

}
